@@ -70,13 +70,14 @@
 // "-virtual-storage" specifies which virtual storage to check for data loss. If not specified,
 // the check is performed for every configured virtual storage.
 //
-// Enable Writes
+// Accept Dataloss
 //
-// The subcommand "enable-writes" re-enables writes to a storage after it has experienced a
-// failover event. Once writes are re-enabled, the new primary's state might diverge from the
-// previous primary, making data recovery attempts more difficult due to possible conflicts.
+// The subcommand "accept-dataloss" allows for accepting data loss in a repository to enable it for
+// writing again. The current version of the repository on the authoritative storage is set to be
+// the latest version and replications to other nodes are scheduled in order to bring them consistent
+// with the new authoritative version.
 //
-//     praefect -config PATH_TO_CONFIG enable-writes -virtual-storage <vstorage>
+//     praefect -config PATH_TO_CONFIG accept-dataloss -virtual-storage <virtual-storage> -relative-path <relative-path> -authoritative-storage <authoritative-storage>
 package main
 
 import (
@@ -226,43 +227,44 @@ func run(cfgs []starter.Config, conf config.Config) error {
 	}
 
 	var queue datastore.ReplicationEventQueue
+	var rs datastore.RepositoryStore
 	if conf.MemoryQueueEnabled {
 		queue = datastore.NewMemoryReplicationEventQueue(conf)
+		rs = datastore.NewMemoryRepositoryStore(conf.StorageNames())
 	} else {
 		queue = datastore.NewPostgresReplicationEventQueue(db)
+		rs = datastore.NewPostgresRepositoryStore(db, conf.StorageNames())
 	}
 
-	nodeManager, err := nodes.NewManager(logger, conf, db, queue, nodeLatencyHistogram)
+	nodeManager, err := nodes.NewManager(logger, conf, db, rs, nodeLatencyHistogram)
 	if err != nil {
 		return err
 	}
-	nodeManager.Start(1*time.Second, 3*time.Second)
-
-	transactionCounterMetric, err := metrics.RegisterTransactionCounter()
-	if err != nil {
-		return err
-	}
-
-	transactionDelayMetric, err := metrics.RegisterTransactionDelay(conf.Prometheus)
-	if err != nil {
-		return err
-	}
-
-	transactionManager := transactions.NewManager(
-		transactions.WithCounterMetric(transactionCounterMetric),
-		transactions.WithDelayMetric(transactionDelayMetric),
-	)
+	nodeManager.Start(conf.Failover.BootstrapInterval.Duration(), conf.Failover.MonitorInterval.Duration())
+	logger.Info("background started: gitaly nodes health monitoring")
 
 	var (
 		// top level server dependencies
-		coordinator = praefect.NewCoordinator(queue, nodeManager, transactionManager, conf, protoregistry.GitalyProtoPreregistered)
-		repl        = praefect.NewReplMgr(
+		transactionManager = transactions.NewManager(conf)
+
+		coordinator = praefect.NewCoordinator(
+			queue,
+			rs,
+			nodeManager,
+			transactionManager,
+			conf,
+			protoregistry.GitalyProtoPreregistered,
+		)
+
+		repl = praefect.NewReplMgr(
 			logger,
 			conf.VirtualStorageNames(),
 			queue,
+			rs,
 			nodeManager,
 			praefect.WithDelayMetric(delayMetric),
 			praefect.WithLatencyMetric(latencyMetric),
+			praefect.WithDequeueBatchSize(conf.Replication.BatchSize),
 		)
 		srvFactory = praefect.NewServerFactory(
 			conf,
@@ -271,11 +273,17 @@ func run(cfgs []starter.Config, conf config.Config) error {
 			nodeManager,
 			transactionManager,
 			queue,
+			rs,
 			protoregistry.GitalyProtoPreregistered,
 		)
 	)
 
-	prometheus.MustRegister(repl)
+	prometheus.MustRegister(
+		transactionManager,
+		coordinator,
+		repl,
+		datastore.NewRepositoryStoreCollector(logger, rs, nodeManager),
+	)
 
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
@@ -316,7 +324,9 @@ func run(cfgs []starter.Config, conf config.Config) error {
 	}
 
 	repl.ProcessBacklog(ctx, praefect.ExpBackoffFunc(1*time.Second, 5*time.Second))
+	logger.Info("background started: processing of the replication events")
 	repl.ProcessStale(ctx, 30*time.Second, time.Minute)
+	logger.Info("background started: processing of the stale replication events")
 
 	return b.Wait(conf.GracefulStopTimeout.Duration())
 }
