@@ -17,6 +17,8 @@ import (
 	"github.com/grpc-ecosystem/go-grpc-middleware/logging/logrus/ctxlogrus"
 	"github.com/sirupsen/logrus"
 	"gitlab.com/gitlab-org/gitaly/internal/config"
+	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
+	"gitlab.com/gitlab-org/labkit/tracing"
 )
 
 const (
@@ -55,6 +57,8 @@ var exportedEnvVars = []string{
 	"NO_PROXY",
 }
 
+var envInjector = tracing.NewEnvInjector()
+
 const (
 	// MaxStderrBytes is at most how many bytes will be written to stderr
 	MaxStderrBytes = 10000 // 10kb
@@ -199,6 +203,7 @@ func New(ctx context.Context, cmd *exec.Cmd, stdin io.Reader, stdout, stderr io.
 
 	// Export env vars
 	cmd.Env = append(env, AllowedEnvironment()...)
+	cmd.Env = envInjector(ctx, cmd.Env)
 
 	// Start the command in its own process group (nice for signalling)
 	cmd.SysProcAttr = &syscall.SysProcAttr{Setpgid: true}
@@ -405,7 +410,8 @@ func (c *Command) logProcessComplete(ctx context.Context, exitCode int) {
 		"command.real_time_ms":   realTime.Seconds() * 1000,
 	})
 
-	if rusage, ok := cmd.ProcessState.SysUsage().(*syscall.Rusage); ok {
+	rusage, ok := cmd.ProcessState.SysUsage().(*syscall.Rusage)
+	if ok {
 		entry = entry.WithFields(logrus.Fields{
 			"command.maxrss":  rusage.Maxrss,
 			"command.inblock": rusage.Inblock,
@@ -413,6 +419,23 @@ func (c *Command) logProcessComplete(ctx context.Context, exitCode int) {
 		})
 	}
 
+	if featureflag.IsEnabled(ctx, featureflag.LogCommandStats) {
+		stats := StatsFromContext(ctx)
+
+		stats.RecordSum("command.count", 1)
+		stats.RecordSum("command.system_time_ms", int(systemTime.Seconds()*1000))
+		stats.RecordSum("command.user_time_ms", int(userTime.Seconds()*1000))
+		stats.RecordSum("command.real_time_ms", int(realTime.Seconds()*1000))
+
+		if ok {
+			stats.RecordMax("command.maxrss", int(rusage.Maxrss))
+			stats.RecordSum("command.inblock", int(rusage.Inblock))
+			stats.RecordSum("command.oublock", int(rusage.Oublock))
+			stats.RecordSum("command.minflt", int(rusage.Minflt))
+			stats.RecordSum("command.majflt", int(rusage.Majflt))
+		}
+	}
+
 	entry.Debug("spawn complete")
 }
 
