@@ -9,12 +9,12 @@ import (
 	"os/exec"
 	"path"
 	"path/filepath"
+	"strings"
 	"testing"
 	"time"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-
 	"gitlab.com/gitlab-org/gitaly/internal/config/sentry"
 )
 
@@ -53,7 +53,7 @@ func TestLoadEmptyConfig(t *testing.T) {
 func TestLoadStorage(t *testing.T) {
 	tmpFile := configFileReader(`[[storage]]
 name = "default"
-path = "/tmp"`)
+path = "/tmp/"`)
 
 	err := Load(tmpFile)
 	assert.NoError(t, err)
@@ -70,6 +70,22 @@ path = "/tmp"`)
 	}
 }
 
+func TestUncleanStoragePaths(t *testing.T) {
+	require.NoError(t, Load(strings.NewReader(`[[storage]]
+name="unclean-path-1"
+path="/tmp/repos1//"
+
+[[storage]]
+name="unclean-path-2"
+path="/tmp/repos2/subfolder/.."
+`)))
+
+	require.Equal(t, []Storage{
+		{Name: "unclean-path-1", Path: "/tmp/repos1"},
+		{Name: "unclean-path-2", Path: "/tmp/repos2"},
+	}, Config.Storages)
+}
+
 func TestLoadMultiStorage(t *testing.T) {
 	tmpFile := configFileReader(`[[storage]]
 name="default"
@@ -77,7 +93,7 @@ path="/tmp/repos1"
 
 [[storage]]
 name="other"
-path="/tmp/repos2"`)
+path="/tmp/repos2/"`)
 
 	err := Load(tmpFile)
 	assert.NoError(t, err)
@@ -642,11 +658,29 @@ func TestLoadGracefulRestartTimeout(t *testing.T) {
 			err := Load(tmpFile)
 			assert.NoError(t, err)
 
-			assert.Equal(t, test.expected, Config.GracefulRestartTimeout)
+			assert.Equal(t, test.expected, Config.GracefulRestartTimeout.Duration())
 		})
 	}
 }
 
+func TestGitlabShellDefaults(t *testing.T) {
+	gitlabShellDir := "/dir"
+	expectedGitlab := Gitlab{
+		SecretFile: filepath.Join(gitlabShellDir, ".gitlab_shell_secret"),
+	}
+
+	expectedHooks := Hooks{
+		CustomHooksDir: filepath.Join(gitlabShellDir, "hooks"),
+	}
+
+	tmpFile := configFileReader(fmt.Sprintf(`[gitlab-shell]
+dir = '%s'`, gitlabShellDir))
+	require.NoError(t, Load(tmpFile))
+
+	require.Equal(t, expectedGitlab, Config.Gitlab)
+	require.Equal(t, expectedHooks, Config.Hooks)
+}
+
 func TestValidateInternalSocketDir(t *testing.T) {
 	defer func(internalSocketDir string) {
 		Config.InternalSocketDir = internalSocketDir
