@@ -18,14 +18,17 @@ func (o *ObjectPool) clone(ctx context.Context, repo *gitalypb.Repository) error
 		return err
 	}
 
-	targetDir := o.FullPath()
-	targetName := path.Base(targetDir)
-	if err != nil {
-		return err
-	}
-
-	cloneArgs := []string{"-C", path.Dir(targetDir), "clone", "--quiet", "--bare", "--local", repoPath, targetName}
-	cmd, err := git.CommandWithoutRepo(ctx, cloneArgs...)
+	cmd, err := git.SafeCmdWithoutRepo(ctx, git.CmdStream{}, nil,
+		git.SubCmd{
+			Name: "clone",
+			Flags: []git.Option{
+				git.Flag{Name: "--quiet"},
+				git.Flag{Name: "--bare"},
+				git.Flag{Name: "--local"},
+			},
+			Args: []string{repoPath, o.FullPath()},
+		},
+	)
 	if err != nil {
 		return err
 	}
