@@ -1,14 +1,34 @@
 package linguist
 
 import (
+	"encoding/json"
+	"os"
 	"path/filepath"
 	"testing"
 
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/internal/config"
-	_ "gitlab.com/gitlab-org/gitaly/internal/testhelper" // Side effect: set up config.Config
+	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 )
 
+func TestMain(m *testing.M) {
+	testhelper.Configure()
+	os.Exit(m.Run())
+}
+
+func TestStatsUnmarshalJSONError(t *testing.T) {
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	// When an error occurs, this used to trigger JSON marshelling of a plain string
+	// the new behaviour shouldn't do that, and return an command error
+	_, err := Stats(ctx, "/var/empty", "deadbeef")
+	require.Error(t, err)
+
+	_, ok := err.(*json.SyntaxError)
+	require.False(t, ok, "expected the error not be a json Syntax Error")
+}
+
 func TestLoadLanguages(t *testing.T) {
 	colorMap = make(map[string]Language)
 	require.NoError(t, LoadColors(config.Config), "load colors")
