@@ -18,6 +18,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/transactions"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper/promtest"
+	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/codes"
 )
@@ -111,11 +112,9 @@ func TestAuthSuccess(t *testing.T) {
 			require.NoError(t, err, tc.desc)
 			defer conn.Close()
 
-			cli := mock.NewSimpleServiceClient(conn)
+			cli := gitalypb.NewServerServiceClient(conn)
 
-			_, err = cli.ServerAccessor(ctx, &mock.SimpleRequest{
-				Value: 1,
-			})
+			_, err = cli.ServerInfo(ctx, &gitalypb.ServerInfoRequest{})
 
 			assert.NoError(t, err, tc.desc)
 		})
@@ -151,10 +150,9 @@ func runServer(t *testing.T, token string, required bool) (*grpc.Server, string,
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						Storage:        "praefect-internal-0",
-						DefaultPrimary: true,
-						Address:        backend,
-						Token:          backendToken,
+						Storage: "praefect-internal-0",
+						Address: backend,
+						Token:   backendToken,
 					},
 				},
 			},
@@ -170,17 +168,19 @@ func runServer(t *testing.T, token string, required bool) (*grpc.Server, string,
 	logEntry := testhelper.DiscardTestEntry(t)
 	queue := datastore.NewMemoryReplicationEventQueue(conf)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, queue, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
 
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(conf)
 
 	registry, err := protoregistry.New(fd)
 	require.NoError(t, err)
 
-	coordinator := NewCoordinator(queue, nodeMgr, txMgr, conf, registry)
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-	srv := NewGRPCServer(conf, logEntry, registry, coordinator.StreamDirector, nodeMgr, txMgr, queue)
+	coordinator := NewCoordinator(queue, rs, nodeMgr, txMgr, conf, registry)
+
+	srv := NewGRPCServer(conf, logEntry, registry, coordinator.StreamDirector, nodeMgr, txMgr, queue, rs)
 
 	serverSocketPath := testhelper.GetTemporaryGitalySocketFileName()
 
