@@ -17,16 +17,34 @@ import (
 type Failover struct {
 	Enabled                  bool            `toml:"enabled"`
 	ElectionStrategy         string          `toml:"election_strategy"`
-	ReadOnlyAfterFailover    bool            `toml:"read_only_after_failover"`
 	ErrorThresholdWindow     config.Duration `toml:"error_threshold_window"`
 	WriteErrorThresholdCount uint32          `toml:"write_error_threshold_count"`
 	ReadErrorThresholdCount  uint32          `toml:"read_error_threshold_count"`
+	// BootstrapInterval allows set a time duration that would be used on startup to make initial health check.
+	// The default value is 1s.
+	BootstrapInterval config.Duration `toml:"bootstrap_interval"`
+	// MonitorInterval allows set a time duration that would be used after bootstrap is completed to execute health checks.
+	// The default value is 3s.
+	MonitorInterval config.Duration `toml:"monitor_interval"`
 }
 
 const sqlFailoverValue = "sql"
 
+// Replication contains replication specific configuration options.
+type Replication struct {
+	// BatchSize controls how many replication jobs to dequeue and lock
+	// in a single call to the database.
+	BatchSize uint `toml:"batch_size"`
+}
+
+// DefaultReplicationConfig returns the default values for replication configuration.
+func DefaultReplicationConfig() Replication {
+	return Replication{BatchSize: 10}
+}
+
 // Config is a container for everything found in the TOML config file
 type Config struct {
+	Replication          Replication       `toml:"replication"`
 	ListenAddr           string            `toml:"listen_addr"`
 	TLSListenAddr        string            `toml:"tls_listen_addr"`
 	SocketPath           string            `toml:"socket_path"`
@@ -54,21 +72,22 @@ type VirtualStorage struct {
 // FromFile loads the config for the passed file path
 func FromFile(filePath string) (Config, error) {
 	conf := &Config{
+		Replication: DefaultReplicationConfig(),
 		// Sets the default Failover, to be overwritten when deserializing the TOML
-		Failover: Failover{Enabled: true, ElectionStrategy: sqlFailoverValue, ReadOnlyAfterFailover: true},
+		Failover: Failover{Enabled: true, ElectionStrategy: sqlFailoverValue},
 	}
 	if _, err := toml.DecodeFile(filePath, conf); err != nil {
 		return Config{}, err
 	}
 
-	conf.setDefaults()
-
 	// TODO: Remove this after failover_enabled has moved under a separate failover section. This is for
 	// backwards compatibility only
 	if conf.FailoverEnabled {
 		conf.Failover.Enabled = true
 	}
 
+	conf.setDefaults()
+
 	return *conf, nil
 }
 
@@ -76,10 +95,8 @@ var (
 	errDuplicateStorage         = errors.New("internal gitaly storages are not unique")
 	errGitalyWithoutAddr        = errors.New("all gitaly nodes must have an address")
 	errGitalyWithoutStorage     = errors.New("all gitaly nodes must have a storage")
-	errMoreThanOnePrimary       = errors.New("only 1 node can be designated as a primary")
 	errNoGitalyServers          = errors.New("no primary gitaly backends configured")
 	errNoListener               = errors.New("no listen address or socket path configured")
-	errNoPrimaries              = errors.New("no primaries designated")
 	errNoVirtualStorages        = errors.New("no virtual storages configured")
 	errStorageAddressDuplicate  = errors.New("multiple storages have the same address")
 	errVirtualStoragesNotUnique = errors.New("virtual storages must have unique names")
@@ -96,6 +113,10 @@ func (c *Config) Validate() error {
 		return errNoVirtualStorages
 	}
 
+	if c.Replication.BatchSize < 1 {
+		return fmt.Errorf("replication batch size was %d but must be >=1", c.Replication.BatchSize)
+	}
+
 	allAddresses := make(map[string]struct{})
 	virtualStorages := make(map[string]struct{}, len(c.VirtualStorages))
 
@@ -114,16 +135,7 @@ func (c *Config) Validate() error {
 		virtualStorages[virtualStorage.Name] = struct{}{}
 
 		storages := make(map[string]struct{}, len(virtualStorage.Nodes))
-		primaries := 0
 		for _, node := range virtualStorage.Nodes {
-			if node.DefaultPrimary {
-				primaries++
-			}
-
-			if primaries > 1 {
-				return fmt.Errorf("virtual storage %q: %w", virtualStorage.Name, errMoreThanOnePrimary)
-			}
-
 			if node.Storage == "" {
 				return fmt.Errorf("virtual storage %q: %w", virtualStorage.Name, errGitalyWithoutStorage)
 			}
@@ -142,10 +154,6 @@ func (c *Config) Validate() error {
 			}
 			allAddresses[node.Address] = struct{}{}
 		}
-
-		if primaries == 0 {
-			return fmt.Errorf("virtual storage %q: %w", virtualStorage.Name, errNoPrimaries)
-		}
 	}
 
 	return nil
@@ -161,6 +169,16 @@ func (c *Config) setDefaults() {
 	if c.GracefulStopTimeout.Duration() == 0 {
 		c.GracefulStopTimeout = config.Duration(time.Minute)
 	}
+
+	if c.Failover.Enabled {
+		if c.Failover.BootstrapInterval.Duration() == 0 {
+			c.Failover.BootstrapInterval = config.Duration(time.Second)
+		}
+
+		if c.Failover.MonitorInterval.Duration() == 0 {
+			c.Failover.MonitorInterval = config.Duration(3 * time.Second)
+		}
+	}
 }
 
 // VirtualStorageNames returns names of all virtual storages configured.
@@ -172,23 +190,37 @@ func (c *Config) VirtualStorageNames() []string {
 	return names
 }
 
+// StorageNames returns storage names by virtual storage.
+func (c *Config) StorageNames() map[string][]string {
+	storages := make(map[string][]string, len(c.VirtualStorages))
+	for _, vs := range c.VirtualStorages {
+		nodes := make([]string, len(vs.Nodes))
+		for i, n := range vs.Nodes {
+			nodes[i] = n.Storage
+		}
+
+		storages[vs.Name] = nodes
+	}
+
+	return storages
+}
+
 // DB holds Postgres client configuration data.
 type DB struct {
-	Host                         string `toml:"host"`
-	Port                         int    `toml:"port"`
-	User                         string `toml:"user"`
-	Password                     string `toml:"password"`
-	DBName                       string `toml:"dbname"`
-	SSLMode                      string `toml:"sslmode"`
-	SSLCert                      string `toml:"sslcert"`
-	SSLKey                       string `toml:"sslkey"`
-	SSLRootCert                  string `toml:"sslrootcert"`
-	StatementTimeoutMilliseconds int    `toml:"default_timeout_ms"`
+	Host        string `toml:"host"`
+	Port        int    `toml:"port"`
+	User        string `toml:"user"`
+	Password    string `toml:"password"`
+	DBName      string `toml:"dbname"`
+	SSLMode     string `toml:"sslmode"`
+	SSLCert     string `toml:"sslcert"`
+	SSLKey      string `toml:"sslkey"`
+	SSLRootCert string `toml:"sslrootcert"`
 }
 
 // ToPQString returns a connection string that can be passed to github.com/lib/pq.
 func (db DB) ToPQString() string {
-	fields := []string{fmt.Sprintf("statement_timeout=%d", db.StatementTimeoutMilliseconds)}
+	var fields []string
 	if db.Port > 0 {
 		fields = append(fields, fmt.Sprintf("port=%d", db.Port))
 	}
