@@ -15,14 +15,14 @@ import (
 
 func TestConfigValidation(t *testing.T) {
 	vs1Nodes := []*Node{
-		{Storage: "internal-1.0", Address: "localhost:23456", Token: "secret-token-1", DefaultPrimary: true},
+		{Storage: "internal-1.0", Address: "localhost:23456", Token: "secret-token-1"},
 		{Storage: "internal-2.0", Address: "localhost:23457", Token: "secret-token-1"},
 		{Storage: "internal-3.0", Address: "localhost:23458", Token: "secret-token-1"},
 	}
 
 	vs2Nodes := []*Node{
 		// storage can have same name as storage in another virtual storage, but all addresses must be unique
-		{Storage: "internal-1.0", Address: "localhost:33456", Token: "secret-token-2", DefaultPrimary: true},
+		{Storage: "internal-1.0", Address: "localhost:33456", Token: "secret-token-2"},
 		{Storage: "internal-2.1", Address: "localhost:33457", Token: "secret-token-2"},
 		{Storage: "internal-3.1", Address: "localhost:33458", Token: "secret-token-2"},
 	}
@@ -35,7 +35,8 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Valid config with ListenAddr",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 					{Name: "secondary", Nodes: vs2Nodes},
@@ -46,6 +47,7 @@ func TestConfigValidation(t *testing.T) {
 			desc: "Valid config with TLSListenAddr",
 			config: Config{
 				TLSListenAddr: "tls://localhost:4321",
+				Replication:   DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 				},
@@ -54,12 +56,24 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Valid config with SocketPath",
 			config: Config{
-				SocketPath: "/tmp/praefect.socket",
+				SocketPath:  "/tmp/praefect.socket",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 				},
 			},
 		},
+		{
+			desc: "Invalid replication batch size",
+			config: Config{
+				SocketPath:  "/tmp/praefect.socket",
+				Replication: Replication{BatchSize: 0},
+				VirtualStorages: []*VirtualStorage{
+					{Name: "default", Nodes: vs1Nodes},
+				},
+			},
+			errMsg: "replication batch size was 0 but must be >=1",
+		},
 		{
 			desc: "No ListenAddr or SocketPath or TLSListenAddr",
 			config: Config{
@@ -69,18 +83,23 @@ func TestConfigValidation(t *testing.T) {
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 				},
+				Replication: DefaultReplicationConfig(),
 			},
 			errMsg: "no listen address or socket path configured",
 		},
 		{
-			desc:   "No virtual storages",
-			config: Config{ListenAddr: "localhost:1234"},
+			desc: "No virtual storages",
+			config: Config{
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
+			},
 			errMsg: "no virtual storages configured",
 		},
 		{
 			desc: "duplicate storage",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{
 						Name: "default",
@@ -93,48 +112,19 @@ func TestConfigValidation(t *testing.T) {
 			},
 			errMsg: `virtual storage "default": internal gitaly storages are not unique`,
 		},
-		{
-			desc: "No designated primaries",
-			config: Config{
-				ListenAddr: "localhost:1234",
-				VirtualStorages: []*VirtualStorage{
-					{Name: "default", Nodes: vs1Nodes[1:]},
-				},
-			},
-			errMsg: `virtual storage "default": no primaries designated`,
-		},
-		{
-			desc: "More than 1 primary",
-			config: Config{
-				ListenAddr: "localhost:1234",
-				VirtualStorages: []*VirtualStorage{
-					{
-						Name: "default",
-						Nodes: append(vs1Nodes,
-							&Node{
-								Storage:        "internal-4",
-								Address:        "localhost:23459",
-								Token:          "secret-token",
-								DefaultPrimary: true,
-							}),
-					},
-				},
-			},
-			errMsg: `virtual storage "default": only 1 node can be designated as a primary`,
-		},
 		{
 			desc: "Node storage has no name",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{
 						Name: "default",
 						Nodes: []*Node{
 							{
-								Storage:        "",
-								Address:        "localhost:23456",
-								Token:          "secret-token-1",
-								DefaultPrimary: true,
+								Storage: "",
+								Address: "localhost:23456",
+								Token:   "secret-token-1",
 							},
 						},
 					},
@@ -145,16 +135,16 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Node storage has no address",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{
 						Name: "default",
 						Nodes: []*Node{
 							{
-								Storage:        "internal",
-								Address:        "",
-								Token:          "secret-token-1",
-								DefaultPrimary: true,
+								Storage: "internal",
+								Address: "",
+								Token:   "secret-token-1",
 							},
 						},
 					},
@@ -165,7 +155,8 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Virtual storage has no name",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "", Nodes: vs1Nodes},
 				},
@@ -175,7 +166,8 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Virtual storage not unique",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 					{Name: "default", Nodes: vs2Nodes},
@@ -186,7 +178,8 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Virtual storage has no nodes",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 					{Name: "secondary", Nodes: nil},
@@ -197,7 +190,8 @@ func TestConfigValidation(t *testing.T) {
 		{
 			desc: "Node storage has address duplicate",
 			config: Config{
-				ListenAddr: "localhost:1234",
+				ListenAddr:  "localhost:1234",
+				Replication: DefaultReplicationConfig(),
 				VirtualStorages: []*VirtualStorage{
 					{Name: "default", Nodes: vs1Nodes},
 					{Name: "secondary", Nodes: append(vs2Nodes, vs1Nodes[1])},
@@ -250,9 +244,8 @@ func TestConfigParsing(t *testing.T) {
 						Name: "praefect",
 						Nodes: []*Node{
 							&Node{
-								Address:        "tcp://gitaly-internal-1.example.com",
-								Storage:        "praefect-internal-1",
-								DefaultPrimary: true,
+								Address: "tcp://gitaly-internal-1.example.com",
+								Storage: "praefect-internal-1",
 							},
 							{
 								Address: "tcp://gitaly-internal-2.example.com",
@@ -281,13 +274,15 @@ func TestConfigParsing(t *testing.T) {
 				},
 				MemoryQueueEnabled:  true,
 				GracefulStopTimeout: config.Duration(30 * time.Second),
+				Replication:         Replication{BatchSize: 1},
 				Failover: Failover{
 					Enabled:                  true,
 					ElectionStrategy:         sqlFailoverValue,
-					ReadOnlyAfterFailover:    true,
 					ErrorThresholdWindow:     config.Duration(20 * time.Second),
 					WriteErrorThresholdCount: 1500,
 					ReadErrorThresholdCount:  100,
+					BootstrapInterval:        config.Duration(1 * time.Second),
+					MonitorInterval:          config.Duration(3 * time.Second),
 				},
 			},
 		},
@@ -296,10 +291,12 @@ func TestConfigParsing(t *testing.T) {
 			filePath: "testdata/config.overwritedefaults.toml",
 			expected: Config{
 				GracefulStopTimeout: config.Duration(time.Minute),
+				Replication:         Replication{BatchSize: 1},
 				Failover: Failover{
-					Enabled:               false,
-					ElectionStrategy:      "local",
-					ReadOnlyAfterFailover: false,
+					Enabled:           false,
+					ElectionStrategy:  "local",
+					BootstrapInterval: config.Duration(5 * time.Second),
+					MonitorInterval:   config.Duration(10 * time.Second),
 				},
 			},
 		},
@@ -308,10 +305,12 @@ func TestConfigParsing(t *testing.T) {
 			filePath: "testdata/config.empty.toml",
 			expected: Config{
 				GracefulStopTimeout: config.Duration(time.Minute),
+				Replication:         Replication{BatchSize: 10},
 				Failover: Failover{
-					Enabled:               true,
-					ElectionStrategy:      sqlFailoverValue,
-					ReadOnlyAfterFailover: true,
+					Enabled:           true,
+					ElectionStrategy:  sqlFailoverValue,
+					BootstrapInterval: config.Duration(time.Second),
+					MonitorInterval:   config.Duration(3 * time.Second),
 				},
 			},
 		},
@@ -336,13 +335,25 @@ func TestVirtualStorageNames(t *testing.T) {
 	require.Equal(t, []string{"praefect-1", "praefect-2"}, conf.VirtualStorageNames())
 }
 
+func TestStorageNames(t *testing.T) {
+	conf := Config{
+		VirtualStorages: []*VirtualStorage{
+			{Name: "virtual-storage-1", Nodes: []*Node{{Storage: "gitaly-1"}, {Storage: "gitaly-2"}}},
+			{Name: "virtual-storage-2", Nodes: []*Node{{Storage: "gitaly-3"}, {Storage: "gitaly-4"}}},
+		}}
+	require.Equal(t, map[string][]string{
+		"virtual-storage-1": {"gitaly-1", "gitaly-2"},
+		"virtual-storage-2": {"gitaly-3", "gitaly-4"},
+	}, conf.StorageNames())
+}
+
 func TestToPQString(t *testing.T) {
 	testCases := []struct {
 		desc string
 		in   DB
 		out  string
 	}{
-		{desc: "empty", in: DB{}, out: "statement_timeout=0 binary_parameters=yes"},
+		{desc: "empty", in: DB{}, out: "binary_parameters=yes"},
 		{
 			desc: "basic example",
 			in: DB{
@@ -356,14 +367,14 @@ func TestToPQString(t *testing.T) {
 				SSLKey:      "/path/to/key",
 				SSLRootCert: "/path/to/root-cert",
 			},
-			out: `statement_timeout=0 port=2345 host=1.2.3.4 user=praefect-user password=secret dbname=praefect_production sslmode=require sslcert=/path/to/cert sslkey=/path/to/key sslrootcert=/path/to/root-cert binary_parameters=yes`,
+			out: `port=2345 host=1.2.3.4 user=praefect-user password=secret dbname=praefect_production sslmode=require sslcert=/path/to/cert sslkey=/path/to/key sslrootcert=/path/to/root-cert binary_parameters=yes`,
 		},
 		{
 			desc: "with spaces and quotes",
 			in: DB{
 				Password: "secret foo'bar",
 			},
-			out: `statement_timeout=0 password=secret\ foo\'bar binary_parameters=yes`,
+			out: `password=secret\ foo\'bar binary_parameters=yes`,
 		},
 	}
 
