@@ -7,6 +7,8 @@ import (
 
 	"github.com/golang/protobuf/proto"
 	"github.com/grpc-ecosystem/go-grpc-middleware/logging/logrus/ctxlogrus"
+	"github.com/prometheus/client_golang/prometheus"
+	"github.com/sirupsen/logrus"
 	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/middleware/metadatahandler"
@@ -21,15 +23,11 @@ import (
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"gitlab.com/gitlab-org/labkit/correlation"
 	"golang.org/x/sync/errgroup"
-	"google.golang.org/grpc/codes"
-	"google.golang.org/grpc/status"
 )
 
-type ReadOnlyStorageError string
-
-func (storage ReadOnlyStorageError) Error() string {
-	return fmt.Sprintf("storage %q is in read-only mode", string(storage))
-}
+// ErrRepositoryReadOnly is returned when the repository is in read-only mode. This happens
+// if the primary does not have the latest changes.
+var ErrRepositoryReadOnly = helper.ErrPreconditionFailedf("repository is in read-only mode")
 
 // getReplicationDetails determines the type of job and additional details based on the method name and incoming message
 func getReplicationDetails(methodName string, m proto.Message) (datastore.ChangeType, datastore.Params, error) {
@@ -60,7 +58,6 @@ func getReplicationDetails(methodName string, m proto.Message) (datastore.Change
 			return "", nil, fmt.Errorf("protocol changed: for method %q expected  message type '%T', got '%T'", methodName, req, m)
 		}
 		return datastore.RepackIncremental, nil, nil
-
 	default:
 		return datastore.UpdateRepo, nil, nil
 	}
@@ -78,28 +75,57 @@ type grpcCall struct {
 // downstream server. The coordinator is thread safe; concurrent calls to
 // register nodes are safe.
 type Coordinator struct {
-	nodeMgr  nodes.Manager
-	txMgr    *transactions.Manager
-	queue    datastore.ReplicationEventQueue
-	registry *protoregistry.Registry
-	conf     config.Config
+	nodeMgr      nodes.Manager
+	txMgr        *transactions.Manager
+	queue        datastore.ReplicationEventQueue
+	rs           datastore.RepositoryStore
+	registry     *protoregistry.Registry
+	conf         config.Config
+	votersMetric *prometheus.HistogramVec
 }
 
 // NewCoordinator returns a new Coordinator that utilizes the provided logger
 func NewCoordinator(
 	queue datastore.ReplicationEventQueue,
+	rs datastore.RepositoryStore,
 	nodeMgr nodes.Manager,
 	txMgr *transactions.Manager,
 	conf config.Config,
 	r *protoregistry.Registry,
 ) *Coordinator {
-	return &Coordinator{
+	maxVoters := 1
+	for _, storage := range conf.VirtualStorages {
+		if len(storage.Nodes) > maxVoters {
+			maxVoters = len(storage.Nodes)
+		}
+	}
+
+	coordinator := &Coordinator{
 		queue:    queue,
+		rs:       rs,
 		registry: r,
 		nodeMgr:  nodeMgr,
 		txMgr:    txMgr,
 		conf:     conf,
+		votersMetric: prometheus.NewHistogramVec(
+			prometheus.HistogramOpts{
+				Name:    "gitaly_praefect_voters_per_transaction_total",
+				Help:    "The number of voters a given transaction was created with",
+				Buckets: prometheus.LinearBuckets(1, 1, maxVoters),
+			},
+			[]string{"virtual_storage"},
+		),
 	}
+
+	return coordinator
+}
+
+func (c *Coordinator) Describe(descs chan<- *prometheus.Desc) {
+	prometheus.DescribeByCollect(c, descs)
+}
+
+func (c *Coordinator) Collect(metrics chan<- prometheus.Metric) {
+	c.votersMetric.Collect(metrics)
 }
 
 func (c *Coordinator) directRepositoryScopedMessage(ctx context.Context, call grpcCall) (*proxy.StreamParameters, error) {
@@ -108,11 +134,21 @@ func (c *Coordinator) directRepositoryScopedMessage(ctx context.Context, call gr
 		return nil, helper.ErrInvalidArgument(fmt.Errorf("repo scoped: %w", err))
 	}
 
+	ctxlogrus.AddFields(ctx, logrus.Fields{
+		"virtual_storage": call.targetRepo.StorageName,
+		"relative_path":   call.targetRepo.RelativePath,
+	})
+
 	if targetRepo.StorageName == "" || targetRepo.RelativePath == "" {
 		return nil, helper.ErrInvalidArgumentf("repo scoped: target repo is invalid")
 	}
 
-	if ctx, err = metadata.InjectPraefectServer(ctx, c.conf); err != nil {
+	praefectServer, err := metadata.PraefectFromConfig(c.conf)
+	if err != nil {
+		return nil, fmt.Errorf("repo scoped: could not create Praefect configuration: %w", err)
+	}
+
+	if ctx, err = praefectServer.Inject(ctx); err != nil {
 		return nil, fmt.Errorf("repo scoped: could not inject Praefect server: %w", err)
 	}
 
@@ -184,6 +220,42 @@ func shouldUseTransaction(ctx context.Context, method string) bool {
 	return true
 }
 
+func (c *Coordinator) registerTransaction(ctx context.Context, primary nodes.Node, secondaries []nodes.Node) (*transactions.Transaction, transactions.CancelFunc, error) {
+	var voters []transactions.Voter
+	var threshold uint
+
+	if featureflag.IsEnabled(ctx, featureflag.ReferenceTransactionsPrimaryWins) {
+		// This voting strategy ensures that transactions always go ahead as long as
+		// the primary doesn't fail because of unrelated reasons. Secondaries' votes do
+		// not matter.
+
+		voters = append(voters, transactions.Voter{
+			Name:  primary.GetStorage(),
+			Votes: 1,
+		})
+		threshold = 1
+
+		for _, node := range secondaries {
+			voters = append(voters, transactions.Voter{
+				Name:  node.GetStorage(),
+				Votes: 0,
+			})
+		}
+	} else {
+		// This voting strategy ensures strong consistency: all nodes will agree on the
+		// same result, but any failed node will abort the transaction.
+		for _, node := range append(secondaries, primary) {
+			voters = append(voters, transactions.Voter{
+				Name:  node.GetStorage(),
+				Votes: 1,
+			})
+			threshold += 1
+		}
+	}
+
+	return c.txMgr.RegisterTransaction(ctx, voters, threshold)
+}
+
 func (c *Coordinator) mutatorStreamParameters(ctx context.Context, call grpcCall, targetRepo *gitalypb.Repository) (*proxy.StreamParameters, error) {
 	virtualStorage := targetRepo.StorageName
 
@@ -192,8 +264,10 @@ func (c *Coordinator) mutatorStreamParameters(ctx context.Context, call grpcCall
 		return nil, fmt.Errorf("mutator call: get shard: %w", err)
 	}
 
-	if c.conf.Failover.ReadOnlyAfterFailover && shard.IsReadOnly {
-		return nil, helper.ErrPreconditionFailed(ReadOnlyStorageError(call.targetRepo.GetStorageName()))
+	if latest, err := c.rs.IsLatestGeneration(ctx, virtualStorage, call.targetRepo.RelativePath, shard.Primary.GetStorage()); err != nil {
+		return nil, fmt.Errorf("check generation: %w", err)
+	} else if !latest {
+		return nil, ErrRepositoryReadOnly
 	}
 
 	primaryMessage, err := rewrittenRepositoryMessage(call.methodInfo, call.msg, shard.Primary.GetStorage())
@@ -217,72 +291,42 @@ func (c *Coordinator) mutatorStreamParameters(ctx context.Context, call grpcCall
 	var secondaryDests []proxy.Destination
 
 	if shouldUseTransaction(ctx, call.fullMethodName) {
-		// Make sure to only let healthy nodes take part in transactions, otherwise we'll be
-		// completely blocked until they come back.
-		healthySecondaries := shard.GetHealthySecondaries()
-
-		var voters []transactions.Voter
-		var threshold uint
-		for _, node := range append(healthySecondaries, shard.Primary) {
-			voters = append(voters, transactions.Voter{
-				Name:  node.GetStorage(),
-				Votes: 1,
-			})
-			threshold += 1
-		}
-
-		transactionID, transactionCleanup, err := c.txMgr.RegisterTransaction(ctx, voters, threshold)
-		if err != nil {
-			return nil, fmt.Errorf("registering transactions: %w", err)
-		}
-		finalizers = append(finalizers, func() error {
-			successByNode, err := transactionCleanup()
-			if err != nil {
-				return err
-			}
-
-			// If the primary node failed the transaction, then
-			// there's no sense in trying to replicate from primary
-			// to secondaries.
-			if !successByNode[shard.Primary.GetStorage()] {
-				return fmt.Errorf("transaction: primary failed vote")
-			}
-
-			failedNodes := make([]nodes.Node, 0, len(successByNode))
-			for node, success := range successByNode {
-				if success {
-					continue
-				}
-
-				secondary, err := shard.GetNode(node)
-				if err != nil {
-					return err
-				}
-
-				failedNodes = append(failedNodes, secondary)
-			}
-
-			if len(failedNodes) == 0 {
-				return nil
-			}
-
-			return c.createReplicaJobs(ctx, virtualStorage, call.targetRepo, shard.Primary, failedNodes, change, params)()
-		})
-
-		injectedCtx, err := metadata.InjectTransaction(ctx, transactionID, shard.Primary.GetStorage(), true)
+		// Only healthy secondaries which are consistent with the primary are allowed to take
+		// part in the transaction. Unhealthy nodes would block the transaction until they come back.
+		// Inconsistent nodes will anyway need repair so including them doesn't make sense. They
+		// also might vote to abort which might unnecessarily fail the transaction.
+		consistentSecondaries, err := c.rs.GetConsistentSecondaries(ctx, virtualStorage, targetRepo.RelativePath, shard.Primary.GetStorage())
 		if err != nil {
 			return nil, err
 		}
 
+		participatingSecondaries := make([]nodes.Node, 0, len(consistentSecondaries))
+		for _, secondary := range shard.GetHealthySecondaries() {
+			if _, ok := consistentSecondaries[secondary.GetStorage()]; ok {
+				participatingSecondaries = append(participatingSecondaries, secondary)
+			}
+		}
+
+		c.votersMetric.WithLabelValues(virtualStorage).Observe(float64(1 + len(participatingSecondaries)))
+
+		transaction, transactionCleanup, err := c.registerTransaction(ctx, shard.Primary, participatingSecondaries)
+		if err != nil {
+			return nil, err
+		}
+
+		injectedCtx, err := metadata.InjectTransaction(ctx, transaction.ID(), shard.Primary.GetStorage(), true)
+		if err != nil {
+			return nil, err
+		}
 		primaryDest.Ctx = helper.IncomingToOutgoing(injectedCtx)
 
-		for _, secondary := range healthySecondaries {
+		for _, secondary := range participatingSecondaries {
 			secondaryMsg, err := rewrittenRepositoryMessage(call.methodInfo, call.msg, secondary.GetStorage())
 			if err != nil {
 				return nil, err
 			}
 
-			injectedCtx, err := metadata.InjectTransaction(ctx, transactionID, secondary.GetStorage(), false)
+			injectedCtx, err := metadata.InjectTransaction(ctx, transaction.ID(), secondary.GetStorage(), false)
 			if err != nil {
 				return nil, err
 			}
@@ -293,8 +337,22 @@ func (c *Coordinator) mutatorStreamParameters(ctx context.Context, call grpcCall
 				Msg:  secondaryMsg,
 			})
 		}
+
+		finalizers = append(finalizers,
+			transactionCleanup, c.createTransactionFinalizer(ctx, transaction, shard, virtualStorage, call.targetRepo, change, params),
+		)
 	} else {
-		finalizers = append(finalizers, c.createReplicaJobs(ctx, virtualStorage, call.targetRepo, shard.Primary, shard.Secondaries, change, params))
+		finalizers = append(finalizers,
+			c.newRequestFinalizer(
+				ctx,
+				virtualStorage,
+				call.targetRepo,
+				shard.Primary.GetStorage(),
+				nil,
+				nodesToStorages(shard.Secondaries),
+				change,
+				params,
+			))
 	}
 
 	reqFinalizer := func() error {
@@ -367,22 +425,113 @@ func (c *Coordinator) StreamDirector(ctx context.Context, fullMethodName string,
 		return sp, nil
 	}
 
-	// TODO: remove the need to handle non repository scoped RPCs. The only remaining one is FindRemoteRepository.
-	// https://gitlab.com/gitlab-org/gitaly/issues/2442. One this issue is resolved, we can explicitly require that
-	// any RPC that gets proxied through praefect must be repository scoped.
-	shard, err := c.nodeMgr.GetShard(c.conf.VirtualStorages[0].Name)
-	if err != nil {
-		if errors.Is(err, nodes.ErrVirtualStorageNotExist) {
-			return nil, status.Errorf(codes.InvalidArgument, err.Error())
-		}
-		return nil, err
+	if mi.Scope == protoregistry.ScopeStorage {
+		return c.directStorageScopedMessage(ctx, mi, m)
 	}
 
-	return proxy.NewStreamParameters(proxy.Destination{
-		Ctx:  helper.IncomingToOutgoing(ctx),
+	// TODO: please refer to https://gitlab.com/gitlab-org/gitaly/-/issues/2974
+	if mi.Scope == protoregistry.ScopeServer {
+		shard, err := c.nodeMgr.GetShard(c.conf.VirtualStorages[0].Name)
+		if err != nil {
+			if errors.Is(err, nodes.ErrVirtualStorageNotExist) {
+				return nil, helper.ErrInvalidArgument(err)
+			}
+			return nil, err
+		}
+
+		return proxy.NewStreamParameters(proxy.Destination{
+			Ctx:  helper.IncomingToOutgoing(ctx),
+			Conn: shard.Primary.GetConnection(),
+			Msg:  payload,
+		}, nil, func() error { return nil }, nil), nil
+	}
+
+	return nil, helper.ErrInternalf("rpc with undefined scope %q", mi.Scope)
+}
+
+func (c *Coordinator) directStorageScopedMessage(ctx context.Context, mi protoregistry.MethodInfo, msg proto.Message) (*proxy.StreamParameters, error) {
+	virtualStorage, err := mi.Storage(msg)
+	if err != nil {
+		return nil, helper.ErrInvalidArgument(err)
+	}
+
+	if virtualStorage == "" {
+		return nil, helper.ErrInvalidArgumentf("storage scoped: target storage is invalid")
+	}
+
+	var ps *proxy.StreamParameters
+	switch mi.Operation {
+	case protoregistry.OpAccessor:
+		ps, err = c.accessorStorageStreamParameters(ctx, mi, msg, virtualStorage)
+	case protoregistry.OpMutator:
+		ps, err = c.mutatorStorageStreamParameters(ctx, mi, msg, virtualStorage)
+	default:
+		err = fmt.Errorf("storage scope: unknown operation type: %v", mi.Operation)
+	}
+	return ps, err
+}
+
+func (c *Coordinator) accessorStorageStreamParameters(ctx context.Context, mi protoregistry.MethodInfo, msg proto.Message, virtualStorage string) (*proxy.StreamParameters, error) {
+	shard, err := c.nodeMgr.GetShard(virtualStorage)
+	if err != nil {
+		if errors.Is(err, nodes.ErrVirtualStorageNotExist) {
+			return nil, helper.ErrInvalidArgument(err)
+		}
+		return nil, helper.ErrInternalf("accessor storage scoped: get shard %q: %w", virtualStorage, err)
+	}
+
+	primaryStorage := shard.Primary.GetStorage()
+
+	b, err := rewrittenStorageMessage(mi, msg, primaryStorage)
+	if err != nil {
+		return nil, helper.ErrInvalidArgument(fmt.Errorf("accessor storage scoped: %w", err))
+	}
+
+	// As this is a read operation it could be routed to another storage (not only primary) if it meets constraints
+	// such as: it is healthy, it belongs to the same virtual storage bundle, etc.
+	// https://gitlab.com/gitlab-org/gitaly/-/issues/2972
+	primaryDest := proxy.Destination{
+		Ctx:  ctx,
 		Conn: shard.Primary.GetConnection(),
-		Msg:  payload,
-	}, nil, func() error { return nil }, nil), nil
+		Msg:  b,
+	}
+
+	return proxy.NewStreamParameters(primaryDest, nil, func() error { return nil }, nil), nil
+}
+
+func (c *Coordinator) mutatorStorageStreamParameters(ctx context.Context, mi protoregistry.MethodInfo, msg proto.Message, virtualStorage string) (*proxy.StreamParameters, error) {
+	shard, err := c.nodeMgr.GetShard(virtualStorage)
+	if err != nil {
+		if errors.Is(err, nodes.ErrVirtualStorageNotExist) {
+			return nil, helper.ErrInvalidArgument(err)
+		}
+		return nil, helper.ErrInternalf("mutator storage scoped: get shard %q: %w", virtualStorage, err)
+	}
+
+	primaryStorage := shard.Primary.GetStorage()
+
+	b, err := rewrittenStorageMessage(mi, msg, primaryStorage)
+	if err != nil {
+		return nil, helper.ErrInvalidArgument(fmt.Errorf("mutator storage scoped: %w", err))
+	}
+
+	primaryDest := proxy.Destination{
+		Ctx:  ctx,
+		Conn: shard.Primary.GetConnection(),
+		Msg:  b,
+	}
+
+	secondaries := shard.GetHealthySecondaries()
+	secondaryDests := make([]proxy.Destination, len(secondaries))
+	for i, secondary := range secondaries {
+		b, err := rewrittenStorageMessage(mi, msg, secondary.GetStorage())
+		if err != nil {
+			return nil, helper.ErrInvalidArgument(fmt.Errorf("mutator storage scoped: %w", err))
+		}
+		secondaryDests[i] = proxy.Destination{Ctx: ctx, Conn: secondary.GetConnection(), Msg: b}
+	}
+
+	return proxy.NewStreamParameters(primaryDest, secondaryDests, func() error { return nil }, nil), nil
 }
 
 func rewrittenRepositoryMessage(mi protoregistry.MethodInfo, m proto.Message, storage string) ([]byte, error) {
@@ -411,6 +560,14 @@ func rewrittenRepositoryMessage(mi protoregistry.MethodInfo, m proto.Message, st
 	return b, nil
 }
 
+func rewrittenStorageMessage(mi protoregistry.MethodInfo, m proto.Message, storage string) ([]byte, error) {
+	if err := mi.SetStorage(m, storage); err != nil {
+		return nil, helper.ErrInvalidArgument(err)
+	}
+
+	return proxy.NewCodec().Marshal(m)
+}
+
 func protoMessage(mi protoregistry.MethodInfo, frame []byte) (proto.Message, error) {
 	m, err := mi.UnmarshalRequestProto(frame)
 	if err != nil {
@@ -420,27 +577,123 @@ func protoMessage(mi protoregistry.MethodInfo, frame []byte) (proto.Message, err
 	return m, nil
 }
 
-func (c *Coordinator) createReplicaJobs(
+func (c *Coordinator) createTransactionFinalizer(
 	ctx context.Context,
+	transaction *transactions.Transaction,
+	shard nodes.Shard,
 	virtualStorage string,
 	targetRepo *gitalypb.Repository,
-	primary nodes.Node,
-	secondaries []nodes.Node,
 	change datastore.ChangeType,
 	params datastore.Params,
 ) func() error {
 	return func() error {
+		successByNode := transaction.State()
+
+		// If no subtransaction happened, then the called RPC may not be aware of
+		// transactions at all. We thus need to assume it changed repository state
+		// and need to create replication jobs.
+		if transaction.CountSubtransactions() == 0 {
+			secondaries := make([]string, 0, len(successByNode))
+			for secondary := range successByNode {
+				if secondary == shard.Primary.GetStorage() {
+					continue
+				}
+				secondaries = append(secondaries, secondary)
+			}
+
+			return c.newRequestFinalizer(
+				ctx, virtualStorage, targetRepo, shard.Primary.GetStorage(),
+				nil, secondaries, change, params)()
+		}
+
+		// If the primary node failed the transaction, then
+		// there's no sense in trying to replicate from primary
+		// to secondaries.
+		if !successByNode[shard.Primary.GetStorage()] {
+			return fmt.Errorf("transaction: primary failed vote")
+		}
+		delete(successByNode, shard.Primary.GetStorage())
+
+		updatedSecondaries := make([]string, 0, len(successByNode))
+		var outdatedSecondaries []string
+
+		for node, success := range successByNode {
+			if success {
+				updatedSecondaries = append(updatedSecondaries, node)
+				continue
+			}
+
+			outdatedSecondaries = append(outdatedSecondaries, node)
+		}
+
+		return c.newRequestFinalizer(
+			ctx, virtualStorage, targetRepo, shard.Primary.GetStorage(),
+			updatedSecondaries, outdatedSecondaries, change, params)()
+	}
+}
+
+func nodesToStorages(nodes []nodes.Node) []string {
+	storages := make([]string, len(nodes))
+	for i, n := range nodes {
+		storages[i] = n.GetStorage()
+	}
+	return storages
+}
+
+func (c *Coordinator) newRequestFinalizer(
+	ctx context.Context,
+	virtualStorage string,
+	targetRepo *gitalypb.Repository,
+	primary string,
+	updatedSecondaries []string,
+	outdatedSecondaries []string,
+	change datastore.ChangeType,
+	params datastore.Params,
+) func() error {
+	return func() error {
+		switch change {
+		case datastore.UpdateRepo:
+			// If this fails, the primary might have changes on it that are not recorded in the database. The secondaries will appear
+			// consistent with the primary but might serve different stale data. Follow-up mutator calls will solve this state although
+			// the primary will be a later generation in the mean while.
+			if err := c.rs.IncrementGeneration(ctx, virtualStorage, targetRepo.GetRelativePath(), primary, updatedSecondaries); err != nil {
+				return fmt.Errorf("increment generation: %w", err)
+			}
+		case datastore.RenameRepo:
+			// Renaming a repository is not idempotent on Gitaly's side. This combined with a failure here results in a problematic state,
+			// where the client receives an error but can't retry the call as the repository has already been moved on the primary.
+			// Ideally the rename RPC should copy the repository instead of moving so the client can retry if this failed.
+			if err := c.rs.RenameRepository(ctx, virtualStorage, targetRepo.GetRelativePath(), primary, params["RelativePath"].(string)); err != nil {
+				if !errors.Is(err, datastore.RepositoryNotExistsError{}) {
+					return fmt.Errorf("rename repository: %w", err)
+				}
+
+				ctxlogrus.Extract(ctx).WithError(err).Info("renamed repository does not have a store entry")
+			}
+		case datastore.DeleteRepo:
+			// If this fails, the repository was already deleted from the primary but we end up still having a record of it in the db.
+			// Ideally we would delete the record from the db first and schedule the repository for deletion later in order to avoid
+			// this problem. Client can reattempt this as deleting a repository is idempotent.
+			if err := c.rs.DeleteRepository(ctx, virtualStorage, targetRepo.GetRelativePath(), primary); err != nil {
+				if !errors.Is(err, datastore.RepositoryNotExistsError{}) {
+					return fmt.Errorf("delete repository: %w", err)
+				}
+
+				ctxlogrus.Extract(ctx).WithError(err).Info("deleted repository does not have a store entry")
+			}
+		}
+
 		correlationID := c.ensureCorrelationID(ctx, targetRepo)
 
 		g, ctx := errgroup.WithContext(ctx)
-		for _, secondary := range secondaries {
+		for _, secondary := range outdatedSecondaries {
 			event := datastore.ReplicationEvent{
 				Job: datastore.ReplicationJob{
 					Change:            change,
 					RelativePath:      targetRepo.GetRelativePath(),
 					VirtualStorage:    virtualStorage,
-					SourceNodeStorage: primary.GetStorage(),
-					TargetNodeStorage: secondary.GetStorage(),
+					SourceNodeStorage: primary,
+					TargetNodeStorage: secondary,
 					Params:            params,
 				},
 				Meta: datastore.Params{metadatahandler.CorrelationIDKey: correlationID},
