@@ -13,6 +13,7 @@ import (
 	"github.com/golang/protobuf/proto"
 	"github.com/golang/protobuf/ptypes/empty"
 	"github.com/sirupsen/logrus"
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/middleware/metadatahandler"
@@ -30,6 +31,7 @@ import (
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/health/grpc_health_v1"
 	"google.golang.org/grpc/metadata"
+	"google.golang.org/grpc/status"
 )
 
 var testLogger = logrus.New()
@@ -44,74 +46,66 @@ func TestSecondaryRotation(t *testing.T) {
 
 func TestStreamDirectorReadOnlyEnforcement(t *testing.T) {
 	for _, tc := range []struct {
-		readOnly        bool
-		readOnlyEnabled bool
-		shouldError     bool
+		desc     string
+		readOnly bool
 	}{
-		{
-			readOnly:        false,
-			readOnlyEnabled: true,
-			shouldError:     false,
-		},
-		{
-			readOnly:        true,
-			readOnlyEnabled: true,
-			shouldError:     true,
-		},
-		{
-			readOnly:        false,
-			readOnlyEnabled: false,
-			shouldError:     false,
-		},
-		{
-			readOnly:        true,
-			readOnlyEnabled: false,
-			shouldError:     false,
-		},
+		{desc: "writable", readOnly: false},
+		{desc: "read-only", readOnly: true},
 	} {
-		t.Run(fmt.Sprintf("read-only: %v, enabled: %v", tc.readOnly, tc.readOnlyEnabled), func(t *testing.T) {
+		t.Run(tc.desc, func(t *testing.T) {
+			const (
+				virtualStorage = "test-virtual-storage"
+				relativePath   = "test-repository"
+				storage        = "test-storage"
+			)
 			conf := config.Config{
-				Failover: config.Failover{ReadOnlyAfterFailover: tc.readOnlyEnabled},
 				VirtualStorages: []*config.VirtualStorage{
 					&config.VirtualStorage{
-						Name: "praefect",
+						Name: virtualStorage,
 						Nodes: []*config.Node{
 							&config.Node{
-								Address:        "tcp://gitaly-primary.example.com",
-								Storage:        "praefect-internal-1",
-								DefaultPrimary: true,
+								Address: "tcp://gitaly-primary.example.com",
+								Storage: storage,
 							},
 						},
 					},
 				},
 			}
 
-			const storageName = "test-storage"
+			ctx, cancel := testhelper.Context()
+			defer cancel()
+
+			rs := datastore.NewMemoryRepositoryStore(nil)
+			require.NoError(t, rs.SetGeneration(ctx, virtualStorage, relativePath, storage, 1))
+			if tc.readOnly {
+				require.NoError(t, rs.SetGeneration(ctx, virtualStorage, relativePath, storage, 0))
+			}
+
 			coordinator := NewCoordinator(
 				datastore.NewMemoryReplicationEventQueue(conf),
-				&nodes.MockManager{GetShardFunc: func(storage string) (nodes.Shard, error) {
+				rs,
+				&nodes.MockManager{GetShardFunc: func(vs string) (nodes.Shard, error) {
+					require.Equal(t, virtualStorage, vs)
 					return nodes.Shard{
-						IsReadOnly: tc.readOnly,
-						Primary:    &nodes.MockNode{StorageName: storageName},
+						Primary: &nodes.MockNode{GetStorageMethod: func() string {
+							return storage
+						}},
 					}, nil
 				}},
-				transactions.NewManager(),
+				transactions.NewManager(conf),
 				conf,
 				protoregistry.GitalyProtoPreregistered,
 			)
 
-			ctx, cancel := testhelper.Context()
-			defer cancel()
-
 			frame, err := proto.Marshal(&gitalypb.CleanupRequest{Repository: &gitalypb.Repository{
-				StorageName:  storageName,
-				RelativePath: "only-for-validation",
+				StorageName:  virtualStorage,
+				RelativePath: relativePath,
 			}})
 			require.NoError(t, err)
 
 			_, err = coordinator.StreamDirector(ctx, "/gitaly.RepositoryService/Cleanup", &mockPeeker{frame: frame})
-			if tc.shouldError {
-				require.True(t, errors.Is(err, ReadOnlyStorageError(storageName)))
+			if tc.readOnly {
+				require.Equal(t, ErrRepositoryReadOnly, err)
 				testhelper.RequireGrpcError(t, err, codes.FailedPrecondition)
 			} else {
 				require.NoError(t, err)
@@ -128,7 +122,7 @@ func TestStreamDirectorMutator(t *testing.T) {
 	defer srv2.Stop()
 
 	primaryAddress, secondaryAddress := "unix://"+gitalySocket0, "unix://"+gitalySocket1
-	primaryNode := &config.Node{Address: primaryAddress, Storage: "praefect-internal-1", DefaultPrimary: true}
+	primaryNode := &config.Node{Address: primaryAddress, Storage: "praefect-internal-1"}
 	secondaryNode := &config.Node{Address: secondaryAddress, Storage: "praefect-internal-2"}
 	conf := config.Config{
 		VirtualStorages: []*config.VirtualStorage{
@@ -157,12 +151,20 @@ func TestStreamDirectorMutator(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queueInterceptor, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
+	nodeMgr.Start(0, time.Hour)
 
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(conf)
 
-	coordinator := NewCoordinator(queueInterceptor, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	coordinator := NewCoordinator(
+		queueInterceptor,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		nodeMgr,
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	frame, err := proto.Marshal(&gitalypb.FetchIntoObjectPoolRequest{
 		Origin:     &targetRepo,
@@ -222,33 +224,83 @@ func TestStreamDirectorMutator(t *testing.T) {
 
 func TestStreamDirectorMutator_Transaction(t *testing.T) {
 	type node struct {
-		primary       bool
-		vote          string
-		shouldSucceed bool
-		shouldGetRepl bool
+		primary           bool
+		vote              string
+		shouldSucceed     bool
+		shouldGetRepl     bool
+		shouldParticipate bool
+		generation        int
 	}
 
 	testcases := []struct {
-		desc  string
-		nodes []node
+		desc            string
+		disableFeatures []featureflag.FeatureFlag
+		nodes           []node
 	}{
 		{
-			desc: "successful vote should not create replication jobs",
+			desc:            "successful vote should not create replication jobs",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
 			nodes: []node{
-				{primary: true, vote: "foobar", shouldSucceed: true, shouldGetRepl: false},
-				{primary: false, vote: "foobar", shouldSucceed: true, shouldGetRepl: false},
-				{primary: false, vote: "foobar", shouldSucceed: true, shouldGetRepl: false},
+				{primary: true, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
 			},
 		},
 		{
 			// Currently, transactions are created such that all nodes need to agree.
 			// This is going to change in the future, but for now let's just test that
 			// we don't get any replication jobs if any node disagrees.
-			desc: "failing vote should not create replication jobs",
+			desc:            "failing vote should not create replication jobs",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
 			nodes: []node{
-				{primary: true, vote: "foobar", shouldSucceed: false, shouldGetRepl: false},
-				{primary: false, vote: "foobar", shouldSucceed: false, shouldGetRepl: false},
-				{primary: false, vote: "barfoo", shouldSucceed: false, shouldGetRepl: false},
+				{primary: true, vote: "foobar", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "foobar", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "barfoo", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+			},
+		},
+		{
+			desc:            "only consistent secondaries should participate",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
+			nodes: []node{
+				{primary: true, vote: "foobar", shouldSucceed: true, shouldParticipate: true, generation: 1},
+				{primary: false, vote: "foobar", shouldSucceed: true, shouldParticipate: true, generation: 1},
+				{shouldParticipate: false, generation: 0},
+				{shouldParticipate: false, generation: datastore.GenerationUnknown},
+			},
+		},
+		{
+			desc:            "secondaries should not participate when primary's generation is unknown",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
+			nodes: []node{
+				{primary: true, vote: "foobar", shouldSucceed: true, shouldParticipate: true, generation: datastore.GenerationUnknown},
+				{shouldParticipate: false, generation: datastore.GenerationUnknown},
+			},
+		},
+		{
+			desc: "primary succeeds with primary-wins strategy",
+			nodes: []node{
+				{primary: true, vote: "foo", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "qux", shouldSucceed: false, shouldGetRepl: true, shouldParticipate: true},
+				{primary: false, vote: "bar", shouldSucceed: false, shouldGetRepl: true, shouldParticipate: true},
+			},
+		},
+		{
+			desc: "only failing secondaries get replication jobs",
+			nodes: []node{
+				{primary: true, vote: "foo", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "qux", shouldSucceed: false, shouldGetRepl: true, shouldParticipate: true},
+				{primary: false, vote: "foo", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+			},
+		},
+		{
+			// If the transaction didn't receive any votes at all, we need to assume
+			// that the RPC wasn't aware of transactions and thus need to schedule
+			// replication jobs.
+			desc:            "unstarted transaction should create replication jobs",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
+			nodes: []node{
+				{primary: true, shouldSucceed: true, shouldGetRepl: false},
+				{primary: false, shouldSucceed: false, shouldGetRepl: true},
 			},
 		},
 	}
@@ -256,11 +308,11 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 	for _, tc := range testcases {
 		t.Run(tc.desc, func(t *testing.T) {
 			storageNodes := make([]*config.Node, 0, len(tc.nodes))
-			for i, node := range tc.nodes {
+			for i := range tc.nodes {
 				socket := testhelper.GetTemporaryGitalySocketFileName()
 				server, _ := testhelper.NewServerWithHealth(t, socket)
 				defer server.Stop()
-				node := &config.Node{Address: "unix://" + socket, Storage: fmt.Sprintf("node-%d", i), DefaultPrimary: node.primary}
+				node := &config.Node{Address: "unix://" + socket, Storage: fmt.Sprintf("node-%d", i)}
 				storageNodes = append(storageNodes, node)
 			}
 
@@ -287,10 +339,14 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 
 			ctx, cancel := testhelper.Context()
 			defer cancel()
-			ctx = featureflag.IncomingCtxWithFeatureFlag(ctx, featureflag.ReferenceTransactions)
 
-			nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, queueInterceptor, promtest.NewMockHistogramVec())
+			for _, feature := range tc.disableFeatures {
+				ctx = featureflag.IncomingCtxWithDisabledFeatureFlag(ctx, feature)
+			}
+
+			nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
 			require.NoError(t, err)
+			nodeMgr.Start(0, time.Hour)
 
 			shard, err := nodeMgr.GetShard(conf.VirtualStorages[0].Name)
 			require.NoError(t, err)
@@ -301,9 +357,19 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 				waitNodeToChangeHealthStatus(ctx, t, node, true)
 			}
 
-			txMgr := transactions.NewManager()
+			txMgr := transactions.NewManager(conf)
 
-			coordinator := NewCoordinator(queueInterceptor, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+			// set up the generations prior to transaction
+			rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+			for i, n := range tc.nodes {
+				if n.generation == datastore.GenerationUnknown {
+					continue
+				}
+
+				require.NoError(t, rs.SetGeneration(ctx, repo.StorageName, repo.RelativePath, storageNodes[i].Storage, n.generation))
+			}
+
+			coordinator := NewCoordinator(queueInterceptor, rs, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
 
 			fullMethod := "/gitaly.SmartHTTPService/PostReceivePack"
 
@@ -321,24 +387,27 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 
 			var voterWaitGroup sync.WaitGroup
 			for i, node := range tc.nodes {
-				voterWaitGroup.Add(1)
+				if node.shouldGetRepl {
+					replicationWaitGroup.Add(1)
+				}
+
+				if !node.shouldParticipate {
+					continue
+				}
 
 				i := i
 				node := node
 
+				voterWaitGroup.Add(1)
 				go func() {
 					defer voterWaitGroup.Done()
 
-					if node.shouldGetRepl {
-						replicationWaitGroup.Add(1)
-					}
-
 					vote := sha1.Sum([]byte(node.vote))
 					err := txMgr.VoteTransaction(ctx, transaction.ID, fmt.Sprintf("node-%d", i), vote[:])
 					if node.shouldSucceed {
-						require.NoError(t, err)
+						assert.NoError(t, err)
 					} else {
-						require.True(t, errors.Is(err, transactions.ErrTransactionVoteFailed))
+						assert.True(t, errors.Is(err, transactions.ErrTransactionVoteFailed))
 					}
 				}()
 			}
@@ -356,7 +425,20 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 			if primaryShouldSucceed {
 				require.NoError(t, err)
 			} else {
-				require.Error(t, err, errors.New("transaction: primary failed vote"))
+				require.Equal(t, errors.New("transaction: primary failed vote"), err)
+			}
+
+			// Nodes that successfully committed should have their generations incremented.
+			// Nodes that did not successfully commit or did not participate should remain on their
+			// existing generation.
+			for i, n := range tc.nodes {
+				gen, err := rs.GetGeneration(ctx, repo.StorageName, repo.RelativePath, storageNodes[i].Storage)
+				require.NoError(t, err)
+				expectedGeneration := n.generation
+				if n.shouldSucceed {
+					expectedGeneration++
+				}
+				require.Equal(t, expectedGeneration, gen)
 			}
 
 			replicationWaitGroup.Wait()
@@ -386,9 +468,8 @@ func TestStreamDirectorAccessor(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					{
-						Address:        gitalyAddress,
-						Storage:        "praefect-internal-1",
-						DefaultPrimary: true,
+						Address: gitalyAddress,
+						Storage: "praefect-internal-1",
 					},
 				},
 			},
@@ -405,15 +486,24 @@ func TestStreamDirectorAccessor(t *testing.T) {
 	ctx, cancel := testhelper.Context()
 	defer cancel()
 
+	ctx = featureflag.IncomingCtxWithDisabledFeatureFlag(ctx, featureflag.DistributedReads)
+
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queue, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Minute)
 
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(conf)
 
-	coordinator := NewCoordinator(queue, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	coordinator := NewCoordinator(
+		queue,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		nodeMgr,
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	frame, err := proto.Marshal(&gitalypb.FindAllBranchesRequest{Repository: &targetRepo})
 	require.NoError(t, err)
@@ -450,9 +540,8 @@ func TestCoordinatorStreamDirector_distributesReads(t *testing.T) {
 	defer srv2.Stop()
 
 	primaryNodeConf := config.Node{
-		Address:        "unix://" + gitalySocket0,
-		Storage:        "gitaly-1",
-		DefaultPrimary: true,
+		Address: "unix://" + gitalySocket0,
+		Storage: "gitaly-1",
 	}
 
 	secondaryNodeConf := config.Node{
@@ -481,17 +570,29 @@ func TestCoordinatorStreamDirector_distributesReads(t *testing.T) {
 
 	ctx, cancel := testhelper.Context()
 	defer cancel()
-	ctx = featureflag.IncomingCtxWithFeatureFlag(ctx, featureflag.DistributedReads)
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queue, promtest.NewMockHistogramVec())
+	repoStore := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+	require.NoError(t, repoStore.IncrementGeneration(ctx, conf.VirtualStorages[0].Name, targetRepo.RelativePath, primaryNodeConf.Storage, nil))
+	generation, err := repoStore.GetGeneration(ctx, conf.VirtualStorages[0].Name, targetRepo.RelativePath, primaryNodeConf.Storage)
+	require.NoError(t, err)
+	require.NoError(t, repoStore.SetGeneration(ctx, conf.VirtualStorages[0].Name, targetRepo.RelativePath, secondaryNodeConf.Storage, generation))
+
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, repoStore, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Minute)
 
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(conf)
 
-	coordinator := NewCoordinator(queue, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	coordinator := NewCoordinator(
+		queue,
+		repoStore,
+		nodeMgr,
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	t.Run("forwards accessor operations", func(t *testing.T) {
 		var primaryChosen int
@@ -647,9 +748,8 @@ func TestAbsentCorrelationID(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						Address:        primaryAddress,
-						Storage:        "praefect-internal-1",
-						DefaultPrimary: true,
+						Address: primaryAddress,
+						Storage: "praefect-internal-1",
 					},
 					&config.Node{
 						Address: secondaryAddress,
@@ -678,11 +778,20 @@ func TestAbsentCorrelationID(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queueInterceptor, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
-	txMgr := transactions.NewManager()
+	nodeMgr.Start(0, time.Hour)
 
-	coordinator := NewCoordinator(queueInterceptor, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	txMgr := transactions.NewManager(conf)
+
+	coordinator := NewCoordinator(
+		queueInterceptor,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		nodeMgr,
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	frame, err := proto.Marshal(&gitalypb.FetchIntoObjectPoolRequest{
 		Origin:     &targetRepo,
@@ -717,9 +826,8 @@ func TestCoordinatorEnqueueFailure(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						Address:        "unix://woof",
-						Storage:        "praefect-internal-1",
-						DefaultPrimary: true,
+						Address: "unix://woof",
+						Storage: "praefect-internal-1",
 					},
 					&config.Node{
 						Address: "unix://meow",
@@ -754,6 +862,9 @@ func TestCoordinatorEnqueueFailure(t *testing.T) {
 	})
 	defer cleanup()
 
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
 	mcli := mock.NewSimpleServiceClient(cc)
 
 	errQ <- nil
@@ -763,12 +874,309 @@ func TestCoordinatorEnqueueFailure(t *testing.T) {
 			StorageName:  conf.VirtualStorages[0].Name,
 		},
 	}
-	_, err = mcli.RepoMutatorUnary(context.Background(), repoReq)
+	_, err = mcli.RepoMutatorUnary(ctx, repoReq)
 	require.NoError(t, err)
 
 	expectErrMsg := "enqueue failed"
 	errQ <- errors.New(expectErrMsg)
-	_, err = mcli.RepoMutatorUnary(context.Background(), repoReq)
+	_, err = mcli.RepoMutatorUnary(ctx, repoReq)
 	require.Error(t, err)
 	require.Equal(t, err.Error(), "rpc error: code = Unknown desc = enqueue replication event: "+expectErrMsg)
 }
+
+func TestStreamDirectorServerScope(t *testing.T) {
+	gz := proto.FileDescriptor("mock.proto")
+	fd, err := protoregistry.ExtractFileDescriptor(gz)
+	require.NoError(t, err)
+
+	registry, err := protoregistry.New(fd)
+	require.NoError(t, err)
+
+	conf := config.Config{VirtualStorages: []*config.VirtualStorage{{Name: "praefect"}, {Name: "another"}}}
+	mgr := &nodes.MockManager{GetShardFunc: func(s string) (nodes.Shard, error) {
+		require.Equal(t, "praefect", s)
+		return nodes.Shard{Primary: &nodes.MockNode{}}, nil
+	}}
+	coordinator := NewCoordinator(
+		nil,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		mgr,
+		nil,
+		conf,
+		registry,
+	)
+
+	fullMethod := "/mock.SimpleService/ServerAccessor"
+	requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
+
+	frame, err := proto.Marshal(&mock.SimpleRequest{})
+	require.NoError(t, err)
+
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	sp, err := coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+	require.NoError(t, err)
+	require.NotNil(t, sp.Primary())
+	require.Empty(t, sp.Secondaries())
+}
+
+func TestStreamDirectorServerScope_Error(t *testing.T) {
+	gz := proto.FileDescriptor("mock.proto")
+	fd, err := protoregistry.ExtractFileDescriptor(gz)
+	require.NoError(t, err)
+
+	registry, err := protoregistry.New(fd)
+	require.NoError(t, err)
+
+	conf := config.Config{VirtualStorages: []*config.VirtualStorage{{Name: "fake"}, {Name: "another"}}}
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+
+	t.Run("unknown storage provided", func(t *testing.T) {
+		mgr := &nodes.MockManager{
+			GetShardFunc: func(s string) (nodes.Shard, error) {
+				require.Equal(t, "fake", s)
+				return nodes.Shard{}, nodes.ErrVirtualStorageNotExist
+			},
+		}
+		coordinator := NewCoordinator(nil, rs, mgr, nil, conf, registry)
+
+		fullMethod := "/mock.SimpleService/ServerAccessor"
+		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
+
+		frame, err := proto.Marshal(&mock.SimpleRequest{})
+		require.NoError(t, err)
+
+		ctx, cancel := testhelper.Context()
+		defer cancel()
+
+		_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+		require.Error(t, err)
+		result, ok := status.FromError(err)
+		require.True(t, ok)
+		require.Equal(t, codes.InvalidArgument, result.Code())
+		require.Equal(t, "virtual storage does not exist", result.Message())
+	})
+
+	t.Run("primary not healthy", func(t *testing.T) {
+		mgr := &nodes.MockManager{
+			GetShardFunc: func(s string) (nodes.Shard, error) {
+				require.Equal(t, "fake", s)
+				return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
+			},
+		}
+		coordinator := NewCoordinator(nil, rs, mgr, nil, conf, registry)
+
+		fullMethod := "/mock.SimpleService/ServerAccessor"
+		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
+
+		frame, err := proto.Marshal(&mock.SimpleRequest{})
+		require.NoError(t, err)
+
+		ctx, cancel := testhelper.Context()
+		defer cancel()
+
+		_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+		require.Error(t, err)
+		require.Equal(t, nodes.ErrPrimaryNotHealthy, err)
+	})
+}
+
+func TestStreamDirectorStorageScope(t *testing.T) {
+	// stubs health-check requests because nodes.NewManager establishes connection on creation
+	gitalySocket0, gitalySocket1 := testhelper.GetTemporaryGitalySocketFileName(), testhelper.GetTemporaryGitalySocketFileName()
+	srv1, _ := testhelper.NewServerWithHealth(t, gitalySocket0)
+	defer srv1.Stop()
+	srv2, _ := testhelper.NewServerWithHealth(t, gitalySocket1)
+	defer srv2.Stop()
+
+	primaryAddress, secondaryAddress := "unix://"+gitalySocket0, "unix://"+gitalySocket1
+	primaryGitaly := &config.Node{Address: primaryAddress, Storage: "gitaly-1"}
+	secondaryGitaly := &config.Node{Address: secondaryAddress, Storage: "gitaly-2"}
+	conf := config.Config{
+		Failover: config.Failover{Enabled: true},
+		VirtualStorages: []*config.VirtualStorage{{
+			Name:  "praefect",
+			Nodes: []*config.Node{primaryGitaly, secondaryGitaly},
+		}}}
+
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+
+	nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	require.NoError(t, err)
+	nodeMgr.Start(0, time.Second)
+	coordinator := NewCoordinator(nil, rs, nodeMgr, nil, conf, protoregistry.GitalyProtoPreregistered)
+
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	t.Run("mutator", func(t *testing.T) {
+		fullMethod := "/gitaly.NamespaceService/RemoveNamespace"
+		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeStorage, protoregistry.OpMutator)
+
+		frame, err := proto.Marshal(&gitalypb.RemoveNamespaceRequest{
+			StorageName: conf.VirtualStorages[0].Name,
+			Name:        "stub",
+		})
+		require.NoError(t, err)
+
+		streamParams, err := coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+		require.NoError(t, err)
+
+		require.Equal(t, primaryAddress, streamParams.Primary().Conn.Target(), "stream director didn't redirect to gitaly storage")
+
+		rewritten := gitalypb.RemoveNamespaceRequest{}
+		require.NoError(t, proto.Unmarshal(streamParams.Primary().Msg, &rewritten))
+		require.Equal(t, primaryGitaly.Storage, rewritten.StorageName, "stream director didn't rewrite storage")
+	})
+
+	t.Run("accessor", func(t *testing.T) {
+		fullMethod := "/gitaly.NamespaceService/NamespaceExists"
+		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeStorage, protoregistry.OpAccessor)
+
+		frame, err := proto.Marshal(&gitalypb.NamespaceExistsRequest{
+			StorageName: conf.VirtualStorages[0].Name,
+			Name:        "stub",
+		})
+		require.NoError(t, err)
+
+		streamParams, err := coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+		require.NoError(t, err)
+
+		require.Equal(t, primaryAddress, streamParams.Primary().Conn.Target(), "stream director didn't redirect to gitaly storage")
+
+		rewritten := gitalypb.RemoveNamespaceRequest{}
+		require.NoError(t, proto.Unmarshal(streamParams.Primary().Msg, &rewritten))
+		require.Equal(t, primaryGitaly.Storage, rewritten.StorageName, "stream director didn't rewrite storage")
+	})
+}
+
+func TestStreamDirectorStorageScopeError(t *testing.T) {
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	t.Run("no storage provided", func(t *testing.T) {
+		mgr := &nodes.MockManager{
+			GetShardFunc: func(s string) (nodes.Shard, error) {
+				require.FailNow(t, "validation of input was not executed")
+				return nodes.Shard{}, assert.AnError
+			},
+		}
+		coordinator := NewCoordinator(
+			nil,
+			datastore.NewMemoryRepositoryStore(nil),
+			mgr,
+			nil,
+			config.Config{},
+			protoregistry.GitalyProtoPreregistered,
+		)
+
+		frame, err := proto.Marshal(&gitalypb.RemoveNamespaceRequest{StorageName: "", Name: "stub"})
+		require.NoError(t, err)
+
+		_, err = coordinator.StreamDirector(ctx, "/gitaly.NamespaceService/RemoveNamespace", &mockPeeker{frame})
+		require.Error(t, err)
+		result, ok := status.FromError(err)
+		require.True(t, ok)
+		require.Equal(t, codes.InvalidArgument, result.Code())
+		require.Equal(t, "storage scoped: target storage is invalid", result.Message())
+	})
+
+	t.Run("unknown storage provided", func(t *testing.T) {
+		mgr := &nodes.MockManager{
+			GetShardFunc: func(s string) (nodes.Shard, error) {
+				require.Equal(t, "fake", s)
+				return nodes.Shard{}, nodes.ErrVirtualStorageNotExist
+			},
+		}
+		coordinator := NewCoordinator(
+			nil,
+			datastore.NewMemoryRepositoryStore(nil),
+			mgr,
+			nil,
+			config.Config{},
+			protoregistry.GitalyProtoPreregistered,
+		)
+
+		frame, err := proto.Marshal(&gitalypb.RemoveNamespaceRequest{StorageName: "fake", Name: "stub"})
+		require.NoError(t, err)
+
+		_, err = coordinator.StreamDirector(ctx, "/gitaly.NamespaceService/RemoveNamespace", &mockPeeker{frame})
+		require.Error(t, err)
+		result, ok := status.FromError(err)
+		require.True(t, ok)
+		require.Equal(t, codes.InvalidArgument, result.Code())
+		require.Equal(t, "virtual storage does not exist", result.Message())
+	})
+
+	t.Run("primary is not healthy", func(t *testing.T) {
+		t.Run("accessor", func(t *testing.T) {
+			mgr := &nodes.MockManager{
+				GetShardFunc: func(s string) (nodes.Shard, error) {
+					require.Equal(t, "fake", s)
+					return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
+				},
+			}
+			coordinator := NewCoordinator(
+				nil,
+				datastore.NewMemoryRepositoryStore(nil),
+				mgr,
+				nil,
+				config.Config{},
+				protoregistry.GitalyProtoPreregistered,
+			)
+
+			fullMethod := "/gitaly.NamespaceService/NamespaceExists"
+			requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeStorage, protoregistry.OpAccessor)
+
+			frame, err := proto.Marshal(&gitalypb.NamespaceExistsRequest{StorageName: "fake", Name: "stub"})
+			require.NoError(t, err)
+
+			_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+			require.Error(t, err)
+			result, ok := status.FromError(err)
+			require.True(t, ok)
+			require.Equal(t, codes.Internal, result.Code())
+			require.Equal(t, `accessor storage scoped: get shard "fake": primary is not healthy`, result.Message())
+		})
+
+		t.Run("mutator", func(t *testing.T) {
+			mgr := &nodes.MockManager{
+				GetShardFunc: func(s string) (nodes.Shard, error) {
+					require.Equal(t, "fake", s)
+					return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
+				},
+			}
+			coordinator := NewCoordinator(
+				nil,
+				datastore.NewMemoryRepositoryStore(nil),
+				mgr,
+				nil,
+				config.Config{},
+				protoregistry.GitalyProtoPreregistered,
+			)
+
+			fullMethod := "/gitaly.NamespaceService/RemoveNamespace"
+			requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeStorage, protoregistry.OpMutator)
+
+			frame, err := proto.Marshal(&gitalypb.RemoveNamespaceRequest{StorageName: "fake", Name: "stub"})
+			require.NoError(t, err)
+
+			_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
+			require.Error(t, err)
+			result, ok := status.FromError(err)
+			require.True(t, ok)
+			require.Equal(t, codes.Internal, result.Code())
+			require.Equal(t, `mutator storage scoped: get shard "fake": primary is not healthy`, result.Message())
+		})
+	})
+}
+
+func requireScopeOperation(t *testing.T, registry *protoregistry.Registry, fullMethod string, scope protoregistry.Scope, op protoregistry.OpType) {
+	t.Helper()
+
+	mi, err := registry.LookupMethod(fullMethod)
+	require.NoError(t, err)
+	require.Equal(t, scope, mi.Scope, "scope doesn't match requested")
+	require.Equal(t, op, mi.Operation, "operation type doesn't match requested")
+}
