@@ -54,10 +54,6 @@ func testConfig(backends int) config.Config {
 			Token:   fmt.Sprintf("%d", i),
 		}
 
-		if i == 0 {
-			n.DefaultPrimary = true
-		}
-
 		nodes = append(nodes, n)
 	}
 	cfg := config.Config{
@@ -72,18 +68,6 @@ func testConfig(backends int) config.Config {
 	return cfg
 }
 
-func assertPrimariesExist(t testing.TB, conf config.Config) {
-	for _, vs := range conf.VirtualStorages {
-		var defaultNode *config.Node
-		for _, n := range vs.Nodes {
-			if n.DefaultPrimary {
-				defaultNode = n
-			}
-		}
-		require.NotNil(t, defaultNode)
-	}
-}
-
 // runPraefectServer runs a praefect server with the provided mock servers.
 // Each mock server is keyed by the corresponding index of the node in the
 // config.Nodes. There must be a 1-to-1 mapping between backend server and
@@ -109,10 +93,9 @@ func noopBackoffFunc() (backoff, backoffReset) {
 type nullNodeMgr struct{}
 
 func (nullNodeMgr) GetShard(virtualStorageName string) (nodes.Shard, error) {
-	return nodes.Shard{}, nil
+	return nodes.Shard{Primary: &nodes.MockNode{}}, nil
 }
 
-func (nullNodeMgr) EnableWrites(ctx context.Context, virtualStorageName string) error { return nil }
 func (nullNodeMgr) GetSyncedNode(ctx context.Context, virtualStorageName, repoPath string) (nodes.Node, error) {
 	return nil, nil
 }
@@ -124,6 +107,7 @@ type buildOptions struct {
 	withAnnotations *protoregistry.Registry
 	withLogger      *logrus.Entry
 	withNodeMgr     nodes.Manager
+	withRepoStore   datastore.RepositoryStore
 }
 
 func withMockBackends(t testing.TB, backends map[string]mock.SimpleServiceServer) func([]*config.VirtualStorage) []testhelper.Cleanup {
@@ -191,7 +175,7 @@ func runPraefectServerWithGitaly(t *testing.T, conf config.Config) (*grpc.Client
 func runPraefectServerWithGitalyWithDatastore(t *testing.T, conf config.Config, queue datastore.ReplicationEventQueue) (*grpc.ClientConn, *grpc.Server, testhelper.Cleanup) {
 	return runPraefectServer(t, conf, buildOptions{
 		withQueue:    queue,
-		withTxMgr:    transactions.NewManager(),
+		withTxMgr:    transactions.NewManager(conf),
 		withBackends: withRealGitalyShared(t),
 	})
 }
@@ -200,27 +184,32 @@ func defaultQueue(conf config.Config) datastore.ReplicationEventQueue {
 	return datastore.NewMemoryReplicationEventQueue(conf)
 }
 
-func defaultTxMgr() *transactions.Manager {
-	return transactions.NewManager()
+func defaultTxMgr(conf config.Config) *transactions.Manager {
+	return transactions.NewManager(conf)
 }
 
-func defaultNodeMgr(t testing.TB, conf config.Config, queue datastore.ReplicationEventQueue) nodes.Manager {
-	nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, queue, promtest.NewMockHistogramVec())
+func defaultNodeMgr(t testing.TB, conf config.Config, rs datastore.RepositoryStore) nodes.Manager {
+	nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, rs, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
-	nodeMgr.Start(1*time.Millisecond, 5*time.Millisecond)
+	nodeMgr.Start(0, time.Hour)
 	return nodeMgr
 }
 
-func runPraefectServer(t testing.TB, conf config.Config, opt buildOptions) (*grpc.ClientConn, *grpc.Server, testhelper.Cleanup) {
-	assertPrimariesExist(t, conf)
+func defaultRepoStore(conf config.Config) datastore.RepositoryStore {
+	return datastore.NewMemoryRepositoryStore(conf.StorageNames())
+}
 
+func runPraefectServer(t testing.TB, conf config.Config, opt buildOptions) (*grpc.ClientConn, *grpc.Server, testhelper.Cleanup) {
 	var cleanups []testhelper.Cleanup
 
 	if opt.withQueue == nil {
 		opt.withQueue = defaultQueue(conf)
 	}
+	if opt.withRepoStore == nil {
+		opt.withRepoStore = defaultRepoStore(conf)
+	}
 	if opt.withTxMgr == nil {
-		opt.withTxMgr = defaultTxMgr()
+		opt.withTxMgr = defaultTxMgr(conf)
 	}
 	if opt.withBackends != nil {
 		cleanups = append(cleanups, opt.withBackends(conf.VirtualStorages)...)
@@ -232,11 +221,14 @@ func runPraefectServer(t testing.TB, conf config.Config, opt buildOptions) (*grp
 		opt.withLogger = log.Default()
 	}
 	if opt.withNodeMgr == nil {
-		opt.withNodeMgr = defaultNodeMgr(t, conf, opt.withQueue)
+		opt.withNodeMgr = defaultNodeMgr(t, conf, opt.withRepoStore)
 	}
 
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+
 	coordinator := NewCoordinator(
 		opt.withQueue,
+		rs,
 		opt.withNodeMgr,
 		opt.withTxMgr,
 		conf,
@@ -248,10 +240,11 @@ func runPraefectServer(t testing.TB, conf config.Config, opt buildOptions) (*grp
 		opt.withLogger,
 		conf.VirtualStorageNames(),
 		opt.withQueue,
+		rs,
 		opt.withNodeMgr,
 	)
 
-	prf := NewGRPCServer(conf, opt.withLogger, protoregistry.GitalyProtoPreregistered, coordinator.StreamDirector, opt.withNodeMgr, opt.withTxMgr, opt.withQueue)
+	prf := NewGRPCServer(conf, opt.withLogger, protoregistry.GitalyProtoPreregistered, coordinator.StreamDirector, opt.withNodeMgr, opt.withTxMgr, opt.withQueue, rs)
 
 	listener, port := listenAvailPort(t)
 	t.Logf("proxy listening on port %d", port)
@@ -378,6 +371,7 @@ func dialLocalPort(tb testing.TB, port int, backend bool) *grpc.ClientConn {
 func newMockDownstream(tb testing.TB, token string, m mock.SimpleServiceServer) (string, func()) {
 	srv := grpc.NewServer(grpc.UnaryInterceptor(auth.UnaryServerInterceptor(internalauth.Config{Token: token})))
 	mock.RegisterSimpleServiceServer(srv, m)
+	healthpb.RegisterHealthServer(srv, health.NewServer())
 
 	// client to backend service
 	lis, port := listenAvailPort(tb)
