@@ -3,13 +3,13 @@ package nodes
 import (
 	"context"
 	"errors"
+	"fmt"
 	"net"
 	"testing"
 	"time"
 
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/client"
-	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/datastore"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
@@ -25,10 +25,8 @@ type nodeAssertion struct {
 }
 
 type shardAssertion struct {
-	PreviousWritablePrimary *nodeAssertion
-	IsReadOnly              bool
-	Primary                 *nodeAssertion
-	Secondaries             []nodeAssertion
+	Primary     *nodeAssertion
+	Secondaries []nodeAssertion
 }
 
 func toNodeAssertion(n Node) *nodeAssertion {
@@ -51,10 +49,8 @@ func assertShard(t *testing.T, exp shardAssertion, act Shard) {
 	}
 
 	require.Equal(t, exp, shardAssertion{
-		PreviousWritablePrimary: toNodeAssertion(act.PreviousWritablePrimary),
-		IsReadOnly:              act.IsReadOnly,
-		Primary:                 toNodeAssertion(act.Primary),
-		Secondaries:             actSecondaries,
+		Primary:     toNodeAssertion(act.Primary),
+		Secondaries: actSecondaries,
 	})
 }
 
@@ -75,9 +71,11 @@ func TestNodeStatus(t *testing.T) {
 	storageName := "default"
 	cs := newConnectionStatus(config.Node{Storage: storageName}, cc, testhelper.DiscardTestEntry(t), mockHistogramVec)
 
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
 	var expectedLabels [][]string
 	for i := 0; i < healthcheckThreshold; i++ {
-		ctx := context.Background()
 		status, err := cs.CheckHealth(ctx)
 
 		require.NoError(t, err)
@@ -90,7 +88,6 @@ func TestNodeStatus(t *testing.T) {
 
 	healthSvr.SetServingStatus("", grpc_health_v1.HealthCheckResponse_NOT_SERVING)
 
-	ctx := context.Background()
 	status, err := cs.CheckHealth(ctx)
 	require.NoError(t, err)
 	require.False(t, status)
@@ -104,9 +101,8 @@ func TestManagerFailoverDisabledElectionStrategySQL(t *testing.T) {
 		Name: virtualStorageName,
 		Nodes: []*config.Node{
 			{
-				Storage:        primaryStorage,
-				Address:        "unix://" + socket0,
-				DefaultPrimary: true,
+				Storage: primaryStorage,
+				Address: "unix://" + socket0,
 			},
 			{
 				Storage: "praefect-internal-1",
@@ -138,90 +134,38 @@ func TestManagerFailoverDisabledElectionStrategySQL(t *testing.T) {
 	nm.checkShards()
 
 	shard, err = nm.GetShard(virtualStorageName)
-	require.NoError(t, err)
-	require.Equal(t, primaryStorage, shard.Primary.GetStorage())
+	require.Error(t, err)
+	require.Equal(t, ErrPrimaryNotHealthy, err)
 }
 
-func TestPrimaryIsSecond(t *testing.T) {
-	gitalySocket0, gitalySocket1 := testhelper.GetTemporaryGitalySocketFileName(), testhelper.GetTemporaryGitalySocketFileName()
-	_, healthSrv0 := testhelper.NewServerWithHealth(t, gitalySocket0)
-	_, healthSrv1 := testhelper.NewServerWithHealth(t, gitalySocket1)
-	healthSrv0.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
-	healthSrv1.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
+func TestDialWithUnhealthyNode(t *testing.T) {
+	primaryLn, err := net.Listen("unix", testhelper.GetTemporaryGitalySocketFileName())
+	require.NoError(t, err)
 
-	virtualStorages := []*config.VirtualStorage{
-		{
-			Name: "virtual-storage-0",
-			Nodes: []*config.Node{
-				{
-					Storage:        "praefect-internal-0",
-					Address:        "unix://" + gitalySocket0,
-					DefaultPrimary: false,
-				},
-				{
-					Storage:        "praefect-internal-1",
-					Address:        "unix://" + gitalySocket1,
-					DefaultPrimary: true,
-				},
-			},
-		},
-	}
+	primaryAddress := "unix://" + primaryLn.Addr().String()
+	const secondaryAddress = "unix://does-not-exist"
+	const storageName = "default"
 
 	conf := config.Config{
-		VirtualStorages: virtualStorages,
-		Failover:        config.Failover{Enabled: false},
-	}
-
-	mockHistogram := promtest.NewMockHistogramVec()
-	nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, mockHistogram)
-	require.NoError(t, err)
-
-	shard, err := nm.GetShard("virtual-storage-0")
-	require.NoError(t, err)
-	require.False(t, shard.IsReadOnly, "new shard should not be read-only")
-
-	require.Equal(t, virtualStorages[0].Nodes[1].Storage, shard.Primary.GetStorage())
-	require.Equal(t, virtualStorages[0].Nodes[1].Address, shard.Primary.GetAddress())
-
-	require.Len(t, shard.Secondaries, 1)
-	require.Equal(t, virtualStorages[0].Nodes[0].Storage, shard.Secondaries[0].GetStorage())
-	require.Equal(t, virtualStorages[0].Nodes[0].Address, shard.Secondaries[0].GetAddress())
-}
-
-func TestBlockingDial(t *testing.T) {
-	storageName := "default"
-	praefectSocket := testhelper.GetTemporaryGitalySocketFileName()
-	socketName := "unix://" + praefectSocket
-
-	gitalySocket := testhelper.GetTemporaryGitalySocketFileName()
-
-	lis, err := net.Listen("unix", gitalySocket)
-	require.NoError(t, err)
-
-	conf := config.Config{
-		SocketPath: socketName,
 		VirtualStorages: []*config.VirtualStorage{
 			{
 				Name: storageName,
 				Nodes: []*config.Node{
 					{
-						Storage:        "internal-storage",
-						Address:        "unix://" + gitalySocket,
-						DefaultPrimary: true,
+						Storage: "starts",
+						Address: primaryAddress,
+					},
+					{
+						Storage: "never-starts",
+						Address: secondaryAddress,
 					},
 				},
 			},
 		},
-		Failover: config.Failover{Enabled: true},
 	}
 
-	// simulate gitaly node starting up later
-	go func() {
-		time.Sleep(healthcheckTimeout + 10*time.Millisecond)
-
-		_, healthSrv0 := testhelper.NewHealthServerWithListener(t, lis)
-		healthSrv0.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
-	}()
+	srv, _ := testhelper.NewHealthServerWithListener(t, primaryLn)
+	defer srv.Stop()
 
 	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
@@ -230,8 +174,68 @@ func TestBlockingDial(t *testing.T) {
 
 	shard, err := mgr.GetShard(storageName)
 	require.NoError(t, err)
-	require.Equal(t, "internal-storage", shard.Primary.GetStorage())
-	require.Empty(t, shard.Secondaries)
+	assertShard(t, shardAssertion{
+		Primary:     &nodeAssertion{Storage: "starts", Address: primaryAddress},
+		Secondaries: []nodeAssertion{{Storage: "never-starts", Address: secondaryAddress}},
+	}, shard)
+}
+
+func TestGetPrimaries(t *testing.T) {
+	primarySocket := testhelper.GetTemporaryGitalySocketFileName()
+
+	conf := config.Config{
+		VirtualStorages: []*config.VirtualStorage{
+			{
+				Name: "healthy-primary",
+				Nodes: []*config.Node{
+					{
+						Storage: "primary",
+						Address: "unix://" + primarySocket,
+					},
+				},
+			},
+			{
+				Name: "unhealthy-primary",
+				Nodes: []*config.Node{
+					{
+						Storage: "primary",
+						Address: "unix://does-not-exist",
+					},
+				},
+			},
+		},
+		Failover: config.Failover{Enabled: true},
+	}
+
+	primaryListener, err := net.Listen("unix", primarySocket)
+	require.NoError(t, err)
+
+	primarySrv, _ := testhelper.NewHealthServerWithListener(t, primaryListener)
+	defer primarySrv.Stop()
+
+	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	require.NoError(t, err)
+
+	for i := 0; i < healthcheckThreshold; i++ {
+		mgr.checkShards()
+	}
+
+	shard, err := mgr.GetShard("healthy-primary")
+	require.NoError(t, err)
+	assertShard(t, shardAssertion{
+		Primary:     &nodeAssertion{Storage: "primary", Address: "unix://" + primaryListener.Addr().String()},
+		Secondaries: []nodeAssertion{},
+	}, shard)
+
+	_, err = mgr.GetShard("unhealthy-primary")
+	require.Equal(t, ErrPrimaryNotHealthy, err)
+
+	primaries, err := mgr.GetPrimaries(context.Background())
+	require.NoError(t, err)
+	require.Equal(t, map[string]string{
+		"healthy-primary":   "primary",
+		"unhealthy-primary": "",
+	}, primaries)
 }
 
 func TestNodeManager(t *testing.T) {
@@ -243,9 +247,8 @@ func TestNodeManager(t *testing.T) {
 	defer srv1.Stop()
 
 	node1 := &config.Node{
-		Storage:        "praefect-internal-0",
-		Address:        "unix://" + internalSocket0,
-		DefaultPrimary: true,
+		Storage: "praefect-internal-0",
+		Address: "unix://" + internalSocket0,
 	}
 
 	node2 := &config.Node{
@@ -262,7 +265,7 @@ func TestNodeManager(t *testing.T) {
 
 	confWithFailover := config.Config{
 		VirtualStorages: virtualStorages,
-		Failover:        config.Failover{Enabled: true, ReadOnlyAfterFailover: true},
+		Failover:        config.Failover{Enabled: true},
 	}
 	confWithoutFailover := config.Config{
 		VirtualStorages: virtualStorages,
@@ -276,8 +279,9 @@ func TestNodeManager(t *testing.T) {
 	nmWithoutFailover, err := NewManager(testhelper.DiscardTestEntry(t), confWithoutFailover, nil, nil, mockHistogram)
 	require.NoError(t, err)
 
-	nm.Start(1*time.Millisecond, 5*time.Second)
-	nmWithoutFailover.Start(1*time.Millisecond, 5*time.Second)
+	// monitoring period set to 1 hour as we execute health checks by hands in this test
+	nm.Start(0, time.Hour)
+	nmWithoutFailover.Start(0, time.Hour)
 
 	shardWithoutFailover, err := nmWithoutFailover.GetShard("virtual-storage-0")
 	require.NoError(t, err)
@@ -298,6 +302,7 @@ func TestNodeManager(t *testing.T) {
 	checkShards := func(count int) {
 		for i := 0; i < count; i++ {
 			nm.checkShards()
+			nmWithoutFailover.checkShards()
 		}
 	}
 
@@ -309,31 +314,19 @@ func TestNodeManager(t *testing.T) {
 		require.Contains(t, labelsCalled, []string{node.Storage})
 	}
 
+	// since the failover is disabled the attempt to get a shard with unhealthy primary fails
+	_, err = nmWithoutFailover.GetShard("virtual-storage-0")
+	require.Error(t, err)
+	require.Equal(t, ErrPrimaryNotHealthy, err)
+
 	// since the primary is unhealthy, we expect checkShards to demote primary to secondary, and promote the healthy
 	// secondary to primary
-
-	shardWithoutFailover, err = nmWithoutFailover.GetShard("virtual-storage-0")
-	require.NoError(t, err)
-
 	shard, err = nm.GetShard("virtual-storage-0")
 	require.NoError(t, err)
-
-	// shard without failover and shard with failover should not be the same
-	require.NotEqual(t, shardWithoutFailover.Primary.GetStorage(), shard.Primary.GetStorage())
-	require.NotEqual(t, shardWithoutFailover.Primary.GetAddress(), shard.Primary.GetAddress())
-	require.NotEqual(t, shardWithoutFailover.Secondaries[0].GetStorage(), shard.Secondaries[0].GetStorage())
-	require.NotEqual(t, shardWithoutFailover.Secondaries[0].GetAddress(), shard.Secondaries[0].GetAddress())
-
-	// shard without failover should still match the config
-	assertShard(t, initialState, shardWithoutFailover)
-
 	// shard with failover should have promoted a secondary to primary and demoted the primary to a secondary
 	assertShard(t, shardAssertion{
-		// previous write enabled primary should have been recorded
-		PreviousWritablePrimary: &nodeAssertion{node1.Storage, node1.Address},
-		IsReadOnly:              true,
-		Primary:                 &nodeAssertion{node2.Storage, node2.Address},
-		Secondaries:             []nodeAssertion{{node1.Storage, node1.Address}},
+		Primary:     &nodeAssertion{node2.Storage, node2.Address},
+		Secondaries: []nodeAssertion{{node1.Storage, node1.Address}},
 	}, shard)
 
 	// failing back to the original primary
@@ -345,23 +338,8 @@ func TestNodeManager(t *testing.T) {
 	require.NoError(t, err)
 
 	assertShard(t, shardAssertion{
-		// previous rw primary is now the primary again, field remains the same
-		// as node2 was not write enabled
-		PreviousWritablePrimary: &nodeAssertion{node1.Storage, node1.Address},
-		IsReadOnly:              true,
-		Primary:                 &nodeAssertion{node1.Storage, node1.Address},
-		Secondaries:             []nodeAssertion{{node2.Storage, node2.Address}},
-	}, shard)
-
-	require.NoError(t, nm.EnableWrites(context.Background(), "virtual-storage-0"))
-	shard, err = nm.GetShard("virtual-storage-0")
-	require.NoError(t, err)
-
-	assertShard(t, shardAssertion{
-		PreviousWritablePrimary: &nodeAssertion{node1.Storage, node1.Address},
-		IsReadOnly:              false,
-		Primary:                 &nodeAssertion{node1.Storage, node1.Address},
-		Secondaries:             []nodeAssertion{{node2.Storage, node2.Address}},
+		Primary:     &nodeAssertion{node1.Storage, node1.Address},
+		Secondaries: []nodeAssertion{{node2.Storage, node2.Address}},
 	}, shard)
 
 	healthSrv0.SetServingStatus("", grpc_health_v1.HealthCheckResponse_UNKNOWN)
@@ -370,224 +348,160 @@ func TestNodeManager(t *testing.T) {
 
 	_, err = nm.GetShard("virtual-storage-0")
 	require.Error(t, err, "should return error since no nodes are healthy")
-	require.Equal(t, ErrPrimaryNotHealthy, nm.EnableWrites(context.Background(), "virtual-storage-0"),
-		"should not be able to enable writes with unhealthy master")
 }
 
 func TestMgr_GetSyncedNode(t *testing.T) {
-	var sockets [4]string
-	var srvs [4]*grpc.Server
-	var healthSrvs [4]*health.Server
-	for i := 0; i < 4; i++ {
-		sockets[i] = testhelper.GetTemporaryGitalySocketFileName()
-		srvs[i], healthSrvs[i] = testhelper.NewServerWithHealth(t, sockets[i])
+	const count = 3
+	const virtualStorage = "virtual-storage-0"
+	const repoPath = "path/1"
+
+	var srvs [count]*grpc.Server
+	var healthSrvs [count]*health.Server
+	var nodes [count]*config.Node
+	for i := 0; i < count; i++ {
+		socket := testhelper.GetTemporaryGitalySocketFileName()
+		srvs[i], healthSrvs[i] = testhelper.NewServerWithHealth(t, socket)
 		defer srvs[i].Stop()
-	}
-
-	vs0Primary := "unix://" + sockets[0]
-	vs1Primary := "unix://" + sockets[2]
-	vs1Secondary := "unix://" + sockets[3]
-
-	virtualStorages := []*config.VirtualStorage{
-		{
-			Name: "virtual-storage-0",
-			Nodes: []*config.Node{
-				{
-					Storage:        "gitaly-0",
-					Address:        vs0Primary,
-					DefaultPrimary: true,
-				},
-				{
-					Storage: "gitaly-1",
-					Address: "unix://" + sockets[1],
-				},
-			},
-		},
-		{
-			// second virtual storage needed to check there is no intersections between two even with same storage names
-			Name: "virtual-storage-1",
-			Nodes: []*config.Node{
-				{
-					// same storage name as in other virtual storage is used intentionally
-					Storage:        "gitaly-1",
-					Address:        "unix://" + sockets[2],
-					DefaultPrimary: true,
-				},
-				{
-					Storage: "gitaly-2",
-					Address: vs1Secondary,
-				},
-			},
-		},
+		nodes[i] = &config.Node{Storage: fmt.Sprintf("gitaly-%d", i), Address: "unix://" + socket}
 	}
 
 	conf := config.Config{
-		VirtualStorages: virtualStorages,
-		Failover:        config.Failover{Enabled: true},
+		VirtualStorages: []*config.VirtualStorage{{Name: virtualStorage, Nodes: nodes[:]}},
 	}
 
-	mockHistogram := promtest.NewMockHistogramVec()
-
 	ctx, cancel := testhelper.Context()
 	defer cancel()
 
-	ctx = featureflag.IncomingCtxWithFeatureFlag(ctx, featureflag.DistributedReads)
+	verify := func(failover bool, scenario func(t *testing.T, nm Manager, rs datastore.RepositoryStore)) func(*testing.T) {
+		conf.Failover.Enabled = failover
+		rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-	ackEvent := func(queue datastore.ReplicationEventQueue, job datastore.ReplicationJob, state datastore.JobState) datastore.ReplicationEvent {
-		event := datastore.ReplicationEvent{Job: job}
-
-		eevent, err := queue.Enqueue(ctx, event)
+		nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, rs, promtest.NewMockHistogramVec())
 		require.NoError(t, err)
 
-		devents, err := queue.Dequeue(ctx, eevent.Job.VirtualStorage, eevent.Job.TargetNodeStorage, 2)
-		require.NoError(t, err)
-		require.Len(t, devents, 1)
+		for i := range healthSrvs {
+			healthSrvs[i].SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
+		}
+		nm.Start(0, time.Hour)
 
-		acks, err := queue.Acknowledge(ctx, state, []uint64{devents[0].ID})
-		require.NoError(t, err)
-		require.Equal(t, []uint64{devents[0].ID}, acks)
-		return devents[0]
+		return func(t *testing.T) { scenario(t, nm, rs) }
 	}
 
-	verify := func(scenario func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue)) func(*testing.T) {
-		queue := datastore.NewMemoryReplicationEventQueue(conf)
-
-		nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, queue, mockHistogram)
-		require.NoError(t, err)
-
-		nm.Start(time.Duration(0), time.Hour)
-
-		return func(t *testing.T) { scenario(t, nm, queue) }
-	}
-
-	t.Run("unknown virtual storage", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		_, err := nm.GetSyncedNode(ctx, "virtual-storage-unknown", "")
+	t.Run("unknown virtual storage", verify(true, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		_, err := nm.GetSyncedNode(ctx, "virtual-storage-unknown", "stub")
 		require.True(t, errors.Is(err, ErrVirtualStorageNotExist))
 	}))
 
-	t.Run("no replication events", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		node, err := nm.GetSyncedNode(ctx, "virtual-storage-0", "no/matter")
+	t.Run("state is undefined", verify(true, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		node, err := nm.GetSyncedNode(ctx, virtualStorage, "no/matter")
 		require.NoError(t, err)
-		require.Contains(t, []string{vs0Primary, "unix://" + sockets[1]}, node.GetAddress())
+		require.Equal(t, conf.VirtualStorages[0].Nodes[0].Address, node.GetAddress(), "")
 	}))
 
-	t.Run("last replication event is in 'ready'", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		_, err := queue.Enqueue(ctx, datastore.ReplicationEvent{
-			Job: datastore.ReplicationJob{
-				RelativePath:      "path/1",
-				TargetNodeStorage: "gitaly-1",
-				SourceNodeStorage: "gitaly-0",
-				VirtualStorage:    "virtual-storage-0",
-			},
-		})
+	t.Run("multiple storages up to date", verify(true, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		require.NoError(t, rs.IncrementGeneration(ctx, virtualStorage, repoPath, "gitaly-0", nil))
+		generation, err := rs.GetGeneration(ctx, virtualStorage, repoPath, "gitaly-0")
 		require.NoError(t, err)
+		require.NoError(t, rs.SetGeneration(ctx, virtualStorage, repoPath, "gitaly-1", generation))
+		require.NoError(t, rs.SetGeneration(ctx, virtualStorage, repoPath, "gitaly-2", generation))
 
-		node, err := nm.GetSyncedNode(ctx, "virtual-storage-0", "path/1")
-		require.NoError(t, err)
-		require.Equal(t, vs0Primary, node.GetAddress())
+		chosen := map[Node]struct{}{}
+		for i := 0; i < 1000 && len(chosen) < 2; i++ {
+			node, err := nm.GetSyncedNode(ctx, virtualStorage, repoPath)
+			require.NoError(t, err)
+			chosen[node] = struct{}{}
+		}
+		if len(chosen) < 2 {
+			require.FailNow(t, "no distribution in too many attempts")
+		}
 	}))
 
-	t.Run("last replication event is in 'in_progress'", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		vs0Event, err := queue.Enqueue(ctx, datastore.ReplicationEvent{
-			Job: datastore.ReplicationJob{
-				RelativePath:      "path/1",
-				TargetNodeStorage: "gitaly-1",
-				SourceNodeStorage: "gitaly-0",
-				VirtualStorage:    "virtual-storage-0",
-			},
-		})
+	t.Run("single secondary storage up to date but unhealthy", verify(true, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		require.NoError(t, rs.IncrementGeneration(ctx, virtualStorage, repoPath, "gitaly-0", nil))
+		generation, err := rs.GetGeneration(ctx, virtualStorage, repoPath, "gitaly-0")
+		require.NoError(t, err)
+		require.NoError(t, rs.SetGeneration(ctx, virtualStorage, repoPath, "gitaly-1", generation))
+
+		healthSrvs[1].SetServingStatus("", grpc_health_v1.HealthCheckResponse_UNKNOWN)
+
+		shard, err := nm.GetShard(virtualStorage)
 		require.NoError(t, err)
 
-		vs0Events, err := queue.Dequeue(ctx, vs0Event.Job.VirtualStorage, vs0Event.Job.TargetNodeStorage, 100500)
+		gitaly1, err := shard.GetNode("gitaly-1")
 		require.NoError(t, err)
-		require.Len(t, vs0Events, 1)
 
-		node, err := nm.GetSyncedNode(ctx, "virtual-storage-0", "path/1")
+		ok, err := gitaly1.CheckHealth(ctx)
 		require.NoError(t, err)
-		require.Equal(t, vs0Primary, node.GetAddress())
+		require.False(t, ok)
+
+		node, err := nm.GetSyncedNode(ctx, virtualStorage, repoPath)
+		require.NoError(t, err)
+		require.Equal(t, conf.VirtualStorages[0].Nodes[0].Address, node.GetAddress(), "secondary shouldn't be chosen as it is unhealthy")
 	}))
 
-	t.Run("last replication event is in 'failed'", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		vs0Event := ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-1",
-			SourceNodeStorage: "gitaly-0",
-			VirtualStorage:    "virtual-storage-0",
-		}, datastore.JobStateFailed)
-
-		node, err := nm.GetSyncedNode(ctx, vs0Event.Job.VirtualStorage, vs0Event.Job.RelativePath)
+	t.Run("no healthy storages", verify(true, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		require.NoError(t, rs.IncrementGeneration(ctx, virtualStorage, repoPath, "gitaly-0", nil))
+		generation, err := rs.GetGeneration(ctx, virtualStorage, repoPath, "gitaly-0")
 		require.NoError(t, err)
-		require.Equal(t, vs0Primary, node.GetAddress())
+		require.NoError(t, rs.SetGeneration(ctx, virtualStorage, repoPath, "gitaly-1", generation))
+
+		healthSrvs[0].SetServingStatus("", grpc_health_v1.HealthCheckResponse_UNKNOWN)
+		healthSrvs[1].SetServingStatus("", grpc_health_v1.HealthCheckResponse_UNKNOWN)
+
+		shard, err := nm.GetShard(virtualStorage)
+		require.NoError(t, err)
+
+		gitaly0, err := shard.GetNode("gitaly-0")
+		require.NoError(t, err)
+
+		gitaly0OK, err := gitaly0.CheckHealth(ctx)
+		require.NoError(t, err)
+		require.False(t, gitaly0OK)
+
+		gitaly1, err := shard.GetNode("gitaly-1")
+		require.NoError(t, err)
+
+		gitaly1OK, err := gitaly1.CheckHealth(ctx)
+		require.NoError(t, err)
+		require.False(t, gitaly1OK)
+
+		_, err = nm.GetSyncedNode(ctx, virtualStorage, repoPath)
+		require.True(t, errors.Is(err, ErrPrimaryNotHealthy))
 	}))
 
-	t.Run("multiple replication events for same virtual, last is in 'ready'", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		vsEvent0 := ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-1",
-			SourceNodeStorage: "gitaly-0",
-			VirtualStorage:    "virtual-storage-0",
-		}, datastore.JobStateCompleted)
+	t.Run("disabled failover doesn't disable health state", verify(false, func(t *testing.T, nm Manager, rs datastore.RepositoryStore) {
+		require.NoError(t, rs.IncrementGeneration(ctx, virtualStorage, repoPath, "gitaly-0", nil))
+		generation, err := rs.GetGeneration(ctx, virtualStorage, repoPath, "gitaly-0")
+		require.NoError(t, err)
+		require.NoError(t, rs.SetGeneration(ctx, virtualStorage, repoPath, "gitaly-1", generation))
 
-		vsEvent1, err := queue.Enqueue(ctx, datastore.ReplicationEvent{
-			Job: datastore.ReplicationJob{
-				RelativePath:      vsEvent0.Job.RelativePath,
-				TargetNodeStorage: vsEvent0.Job.TargetNodeStorage,
-				SourceNodeStorage: vsEvent0.Job.SourceNodeStorage,
-				VirtualStorage:    vsEvent0.Job.VirtualStorage,
-			},
-		})
+		shard, err := nm.GetShard(virtualStorage)
 		require.NoError(t, err)
 
-		node, err := nm.GetSyncedNode(ctx, vsEvent1.Job.VirtualStorage, vsEvent1.Job.RelativePath)
+		gitaly0, err := shard.GetNode("gitaly-0")
 		require.NoError(t, err)
-		require.Equal(t, vs0Primary, node.GetAddress())
-	}))
 
-	t.Run("same repo path for different virtual storages", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		vs0Event := ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-1",
-			SourceNodeStorage: "gitaly-0",
-			VirtualStorage:    "virtual-storage-0",
-		}, datastore.JobStateDead)
+		require.Equal(t, shard.Primary, gitaly0)
 
-		ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-2",
-			SourceNodeStorage: "gitaly-1",
-			VirtualStorage:    "virtual-storage-1",
-		}, datastore.JobStateCompleted)
-
-		node, err := nm.GetSyncedNode(ctx, vs0Event.Job.VirtualStorage, vs0Event.Job.RelativePath)
+		gitaly0OK, err := gitaly0.CheckHealth(ctx)
 		require.NoError(t, err)
-		require.Equal(t, vs0Primary, node.GetAddress())
-	}))
+		require.True(t, gitaly0OK)
 
-	t.Run("secondary is up to date in multi-virtual setup with processed replication jobs", verify(func(t *testing.T, nm Manager, queue datastore.ReplicationEventQueue) {
-		ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-1",
-			SourceNodeStorage: "gitaly-0",
-			VirtualStorage:    "virtual-storage-0",
-		}, datastore.JobStateCompleted)
-
-		ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/1",
-			TargetNodeStorage: "gitaly-2",
-			SourceNodeStorage: "gitaly-1",
-			VirtualStorage:    "virtual-storage-1",
-		}, datastore.JobStateCompleted)
-
-		vs1Event := ackEvent(queue, datastore.ReplicationJob{
-			RelativePath:      "path/2",
-			TargetNodeStorage: "gitaly-2",
-			SourceNodeStorage: "gitaly-1",
-			VirtualStorage:    "virtual-storage-1",
-		}, datastore.JobStateCompleted)
-
-		node, err := nm.GetSyncedNode(ctx, vs1Event.Job.VirtualStorage, vs1Event.Job.RelativePath)
+		gitaly1, err := shard.GetNode("gitaly-1")
 		require.NoError(t, err)
-		require.Contains(t, []string{vs1Primary, vs1Secondary}, node.GetAddress(), "should be one of the secondaries or the primary")
+
+		gitaly1OK, err := gitaly1.CheckHealth(ctx)
+		require.NoError(t, err)
+		require.True(t, gitaly1OK)
+
+		healthSrvs[0].SetServingStatus("", grpc_health_v1.HealthCheckResponse_UNKNOWN)
+		gitaly0OK, err = gitaly0.CheckHealth(ctx)
+		require.NoError(t, err)
+		require.False(t, gitaly0OK, "primary should be unhealthy")
+
+		_, err = nm.GetSyncedNode(ctx, virtualStorage, repoPath)
+		require.Error(t, err)
+		require.True(t, errors.Is(err, ErrPrimaryNotHealthy))
 	}))
 }
 
@@ -609,7 +523,7 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	require.NoError(t, err)
 	defer func() { require.NoError(t, clientConn.Close()) }()
 
-	node := config.Node{Storage: "gitaly-0", Address: address, DefaultPrimary: true}
+	node := config.Node{Storage: "gitaly-0", Address: address}
 
 	ctx, cancel := testhelper.Context()
 	defer cancel()
