@@ -19,14 +19,19 @@ func (m *MockManager) GetShard(storage string) (Shard, error) {
 // for parametrizing behavior.
 type MockNode struct {
 	Node
-	StorageName string
-	Conn        *grpc.ClientConn
+	GetStorageMethod func() string
+	Conn             *grpc.ClientConn
+	Healthy          bool
 }
 
-func (m *MockNode) GetStorage() string { return m.StorageName }
+func (m *MockNode) GetStorage() string { return m.GetStorageMethod() }
 
 func (m *MockNode) GetConnection() *grpc.ClientConn { return m.Conn }
 
 func (m *MockNode) GetAddress() string { return "" }
 
 func (m *MockNode) GetToken() string { return "" }
+
+func (m *MockNode) IsHealthy() bool {
+	return m.Healthy
+}
