@@ -10,6 +10,7 @@ import (
 	"github.com/sirupsen/logrus"
 	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/middleware/metadatahandler"
+	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/datastore"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/metrics"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes"
@@ -38,6 +39,7 @@ type Replicator interface {
 
 type defaultReplicator struct {
 	log *logrus.Entry
+	rs  datastore.RepositoryStore
 }
 
 func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.ReplicationEvent, sourceCC, targetCC *grpc.ClientConn) error {
@@ -51,6 +53,17 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 		RelativePath: event.Job.RelativePath,
 	}
 
+	generation, err := dr.rs.GetReplicatedGeneration(ctx, event.Job.VirtualStorage, event.Job.RelativePath, event.Job.SourceNodeStorage, event.Job.TargetNodeStorage)
+	if err != nil {
+		// Later generation might have already been replicated by an earlier replication job. If that's the case,
+		// we'll simply acknowledge the job. This also prevents accidental downgrades from happening.
+		if errors.Is(err, datastore.DowngradeAttemptedError{}) {
+			return nil
+		}
+
+		return fmt.Errorf("get replicated generation: %w", err)
+	}
+
 	targetRepositoryClient := gitalypb.NewRepositoryServiceClient(targetCC)
 
 	if _, err := targetRepositoryClient.ReplicateRepository(ctx, &gitalypb.ReplicateRepositoryRequest{
@@ -101,6 +114,15 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 		}).Error("checksums do not match")
 	}
 
+	if generation != datastore.GenerationUnknown {
+		return dr.rs.SetGeneration(ctx,
+			event.Job.VirtualStorage,
+			event.Job.RelativePath,
+			event.Job.TargetNodeStorage,
+			generation,
+		)
+	}
+
 	return nil
 }
 
@@ -112,11 +134,23 @@ func (dr defaultReplicator) Destroy(ctx context.Context, event datastore.Replica
 
 	repoSvcClient := gitalypb.NewRepositoryServiceClient(targetCC)
 
-	_, err := repoSvcClient.RemoveRepository(ctx, &gitalypb.RemoveRepositoryRequest{
+	if _, err := repoSvcClient.RemoveRepository(ctx, &gitalypb.RemoveRepositoryRequest{
 		Repository: targetRepo,
-	})
+	}); err != nil {
+		return err
+	}
 
-	return err
+	// If the repository was deleted but this fails, we'll know by the repository not having a record in the virtual
+	// storage but having one for the storage. We can later retry the deletion.
+	if err := dr.rs.DeleteRepository(ctx, event.Job.VirtualStorage, event.Job.RelativePath, event.Job.TargetNodeStorage); err != nil {
+		if !errors.Is(err, datastore.RepositoryNotExistsError{}) {
+			return err
+		}
+
+		dr.log.WithError(err).Info("replicated repository delete does not have a store entry")
+	}
+
+	return nil
 }
 
 func (dr defaultReplicator) Rename(ctx context.Context, event datastore.ReplicationEvent, targetCC *grpc.ClientConn) error {
@@ -137,12 +171,26 @@ func (dr defaultReplicator) Rename(ctx context.Context, event datastore.Replicat
 		return fmt.Errorf("parameter 'RelativePath' has unexpected type: %T", relativePath)
 	}
 
-	_, err := repoSvcClient.RenameRepository(ctx, &gitalypb.RenameRepositoryRequest{
+	if _, err := repoSvcClient.RenameRepository(ctx, &gitalypb.RenameRepositoryRequest{
 		Repository:   targetRepo,
 		RelativePath: relativePath,
-	})
+	}); err != nil {
+		return err
+	}
 
-	return err
+	// If the repository was moved but this fails, we'll have a stale record on the storage but it is missing from the
+	// virtual storage. We can later schedule a deletion to fix the situation. The newly named repository's record
+	// will be present once a replication job arrives for it.
+	if err := dr.rs.RenameRepository(ctx,
+		event.Job.VirtualStorage, event.Job.RelativePath, event.Job.TargetNodeStorage, relativePath); err != nil {
+		if !errors.Is(err, datastore.RepositoryNotExistsError{}) {
+			return err
+		}
+
+		dr.log.WithError(err).Info("replicated repository rename does not have a store entry")
+	}
+
+	return nil
 }
 
 func (dr defaultReplicator) GarbageCollect(ctx context.Context, event datastore.ReplicationEvent, targetCC *grpc.ClientConn) error {
@@ -256,6 +304,7 @@ type ReplMgr struct {
 	replLatencyMetric  prommetrics.HistogramVec
 	replDelayMetric    prommetrics.HistogramVec
 	replJobTimeout     time.Duration
+	dequeueBatchSize   uint
 	// whitelist contains the project names of the repos we wish to replicate
 	whitelist map[string]struct{}
 }
@@ -277,14 +326,21 @@ func WithDelayMetric(h prommetrics.HistogramVec) func(*ReplMgr) {
 	}
 }
 
+// WithDequeueBatchSize configures the number of events to dequeue in a single batch.
+func WithDequeueBatchSize(size uint) func(*ReplMgr) {
+	return func(m *ReplMgr) {
+		m.dequeueBatchSize = size
+	}
+}
+
 // NewReplMgr initializes a replication manager with the provided dependencies
 // and options
-func NewReplMgr(log *logrus.Entry, virtualStorages []string, queue datastore.ReplicationEventQueue, nodeMgr nodes.Manager, opts ...ReplMgrOpt) ReplMgr {
+func NewReplMgr(log *logrus.Entry, virtualStorages []string, queue datastore.ReplicationEventQueue, rs datastore.RepositoryStore, nodeMgr nodes.Manager, opts ...ReplMgrOpt) ReplMgr {
 	r := ReplMgr{
 		log:             log.WithField("component", "replication_manager"),
 		queue:           queue,
 		whitelist:       map[string]struct{}{},
-		replicator:      defaultReplicator{log},
+		replicator:      defaultReplicator{log, rs},
 		virtualStorages: virtualStorages,
 		nodeManager:     nodeMgr,
 		replInFlightMetric: prometheus.NewGaugeVec(
@@ -295,6 +351,7 @@ func NewReplMgr(log *logrus.Entry, virtualStorages []string, queue datastore.Rep
 		),
 		replLatencyMetric: prometheus.NewHistogramVec(prometheus.HistogramOpts{}, []string{"type"}),
 		replDelayMetric:   prometheus.NewHistogramVec(prometheus.HistogramOpts{}, []string{"type"}),
+		dequeueBatchSize:  config.DefaultReplicationConfig().BatchSize,
 	}
 
 	for _, opt := range opts {
@@ -421,12 +478,10 @@ func (r ReplMgr) processBacklog(ctx context.Context, b BackoffFunc, virtualStora
 		if err != nil {
 			logger.WithError(err).Error("error when getting primary and secondaries")
 		} else {
-			targetNodes := shard.Secondaries
-			if shard.IsReadOnly {
-				targetNodes = append(targetNodes, shard.Primary)
-			}
-
-			for _, target := range targetNodes {
+			for _, target := range append(shard.Secondaries, shard.Primary) {
+				if !target.IsHealthy() {
+					continue
+				}
 				totalEvents += r.handleNode(ctx, logger, shard, virtualStorage, target)
 			}
 		}
@@ -446,7 +501,7 @@ func (r ReplMgr) processBacklog(ctx context.Context, b BackoffFunc, virtualStora
 }
 
 func (r ReplMgr) handleNode(ctx context.Context, logger logrus.FieldLogger, shard nodes.Shard, virtualStorage string, target nodes.Node) int {
-	events, err := r.queue.Dequeue(ctx, virtualStorage, target.GetStorage(), 10)
+	events, err := r.queue.Dequeue(ctx, virtualStorage, target.GetStorage(), int(r.dequeueBatchSize))
 	if err != nil {
 		logger.WithField(logWithReplTarget, target.GetStorage()).WithError(err).Error("failed to dequeue replication events")
 		return 0
