@@ -40,6 +40,8 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper/promtest"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/health"
+	healthpb "google.golang.org/grpc/health/grpc_health_v1"
 	"google.golang.org/grpc/metadata"
 	"google.golang.org/grpc/reflection"
 )
@@ -76,10 +78,9 @@ func TestProcessReplicationJob(t *testing.T) {
 				Name: "default",
 				Nodes: []*config.Node{
 					&config.Node{
-						Storage:        "default",
-						Address:        srvSocketPath,
-						Token:          gitaly_config.Config.Auth.Token,
-						DefaultPrimary: true,
+						Storage: "default",
+						Address: srvSocketPath,
+						Token:   gitaly_config.Config.Auth.Token,
 					},
 					&config.Node{
 						Storage: backupStorageName,
@@ -124,7 +125,7 @@ func TestProcessReplicationJob(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queue, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
 	nodeMgr.Start(1*time.Millisecond, 5*time.Millisecond)
 
@@ -153,8 +154,10 @@ func TestProcessReplicationJob(t *testing.T) {
 		Message: "a commit",
 	})
 
-	var replicator defaultReplicator
-	replicator.log = entry
+	replicator := defaultReplicator{
+		log: entry,
+		rs:  datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+	}
 
 	var mockReplicationLatencyHistogramVec promtest.MockHistogramVec
 	var mockReplicationDelayHistogramVec promtest.MockHistogramVec
@@ -163,6 +166,7 @@ func TestProcessReplicationJob(t *testing.T) {
 		testhelper.DiscardTestEntry(t),
 		conf.VirtualStorageNames(),
 		queue,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
 		nodeMgr,
 		WithLatencyMetric(&mockReplicationLatencyHistogramVec),
 		WithDelayMetric(&mockReplicationDelayHistogramVec),
@@ -203,9 +207,8 @@ func TestPropagateReplicationJob(t *testing.T) {
 				Name: "default",
 				Nodes: []*config.Node{
 					{
-						Storage:        primaryStorage,
-						Address:        primarySocketPath,
-						DefaultPrimary: true,
+						Storage: primaryStorage,
+						Address: primarySocketPath,
 					},
 					{
 						Storage: secondaryStorage,
@@ -219,17 +222,19 @@ func TestPropagateReplicationJob(t *testing.T) {
 	queue := datastore.NewMemoryReplicationEventQueue(conf)
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, queue, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
-	nodeMgr.Start(1*time.Millisecond, 5*time.Millisecond)
+	nodeMgr.Start(0, time.Hour)
 
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(conf)
 
-	coordinator := NewCoordinator(queue, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-	replmgr := NewReplMgr(logEntry, conf.VirtualStorageNames(), queue, nodeMgr)
+	coordinator := NewCoordinator(queue, rs, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
 
-	prf := NewGRPCServer(conf, logEntry, protoregistry.GitalyProtoPreregistered, coordinator.StreamDirector, nodeMgr, txMgr, queue)
+	replmgr := NewReplMgr(logEntry, conf.VirtualStorageNames(), queue, rs, nodeMgr)
+
+	prf := NewGRPCServer(conf, logEntry, protoregistry.GitalyProtoPreregistered, coordinator.StreamDirector, nodeMgr, txMgr, queue, rs)
 
 	listener, port := listenAvailPort(t)
 	ctx, cancel := testhelper.Context()
@@ -343,6 +348,7 @@ func runMockRepositoryServer(t *testing.T) (*mockRepositoryServer, string, func(
 	mockServer := newMockRepositoryServer()
 
 	gitalypb.RegisterRepositoryServiceServer(server, mockServer)
+	healthpb.RegisterHealthServer(server, health.NewServer())
 	reflection.Register(server)
 
 	go server.Serve(listener)
@@ -421,9 +427,8 @@ func TestProcessBacklog_FailedJobs(t *testing.T) {
 	defer cleanupFn()
 
 	primary := config.Node{
-		Storage:        "default",
-		Address:        "unix://" + primarySocket,
-		DefaultPrimary: true,
+		Storage: "default",
+		Address: "unix://" + primarySocket,
 	}
 
 	secondary := config.Node{
@@ -504,10 +509,17 @@ func TestProcessBacklog_FailedJobs(t *testing.T) {
 
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, queueInterceptor, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
+	nodeMgr.Start(0, time.Hour)
 
-	replMgr := NewReplMgr(logEntry, conf.VirtualStorageNames(), queueInterceptor, nodeMgr)
+	replMgr := NewReplMgr(
+		logEntry,
+		conf.VirtualStorageNames(),
+		queueInterceptor,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		nodeMgr,
+	)
 	replMgr.ProcessBacklog(ctx, noopBackoffFunc)
 
 	select {
@@ -543,9 +555,8 @@ func TestProcessBacklog_Success(t *testing.T) {
 	defer cleanupFn()
 
 	primary := config.Node{
-		Storage:        "default",
-		Address:        "unix://" + primarySocket,
-		DefaultPrimary: true,
+		Storage: "default",
+		Address: "unix://" + primarySocket,
 	}
 
 	secondary := config.Node{
@@ -644,10 +655,17 @@ func TestProcessBacklog_Success(t *testing.T) {
 
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, queueInterceptor, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
+	nodeMgr.Start(0, time.Hour)
 
-	replMgr := NewReplMgr(logEntry, conf.VirtualStorageNames(), queueInterceptor, nodeMgr)
+	replMgr := NewReplMgr(
+		logEntry,
+		conf.VirtualStorageNames(),
+		queueInterceptor,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		nodeMgr,
+	)
 	replMgr.ProcessBacklog(ctx, noopBackoffFunc)
 
 	select {
@@ -663,6 +681,71 @@ func TestProcessBacklog_Success(t *testing.T) {
 	require.True(t, storage.IsGitDirectory(fullNewPath2), "repository must exist at new last RenameRepository location")
 }
 
+func TestReplMgrProcessBacklog_OnlyHealthyNodes(t *testing.T) {
+	conf := config.Config{
+		VirtualStorages: []*config.VirtualStorage{
+			{
+				Name: "default",
+				Nodes: []*config.Node{
+					{Storage: "node-1"},
+					{Storage: "node-2"},
+					{Storage: "node-3"},
+				},
+			},
+		},
+	}
+
+	ctx, cancel := testhelper.Context()
+
+	queueInterceptor := datastore.NewReplicationEventQueueInterceptor(datastore.NewMemoryReplicationEventQueue(conf))
+	queueInterceptor.OnDequeue(func(_ context.Context, virtualStorageName string, storageName string, _ int, _ datastore.ReplicationEventQueue) ([]datastore.ReplicationEvent, error) {
+		select {
+		case <-ctx.Done():
+			return nil, nil
+		default:
+			assert.Equal(t, conf.VirtualStorages[0].Name, virtualStorageName)
+			assert.Equal(t, conf.VirtualStorages[0].Nodes[2].Storage, storageName)
+			cancel()
+			return nil, nil
+		}
+	})
+
+	nodeMgr := &nodes.MockManager{
+		GetShardFunc: func(string) (shard nodes.Shard, err error) {
+			return nodes.Shard{
+				Primary: &nodes.MockNode{
+					Healthy:          true,
+					GetStorageMethod: func() string { return conf.VirtualStorages[0].Nodes[0].Storage },
+				},
+				Secondaries: []nodes.Node{
+					&nodes.MockNode{
+						GetStorageMethod: func() string {
+							assert.FailNow(t, "as this node is not healthy this method should not be called")
+							return conf.VirtualStorages[0].Nodes[1].Storage
+						},
+						Healthy: false,
+					},
+					&nodes.MockNode{
+						Healthy:          true,
+						GetStorageMethod: func() string { return conf.VirtualStorages[0].Nodes[2].Storage },
+					},
+				},
+			}, nil
+		},
+	}
+
+	replMgr := NewReplMgr(testhelper.DiscardTestEntry(t), conf.VirtualStorageNames(), queueInterceptor, nil, nodeMgr)
+	replMgr.ProcessBacklog(ctx, noopBackoffFunc)
+
+	select {
+	case <-ctx.Done():
+		// completed by scenario
+	case <-time.After(30 * time.Second):
+		// strongly depends on the processing capacity
+		t.Fatal("time limit expired for job to complete")
+	}
+}
+
 type mockReplicator struct {
 	Replicator
 	ReplicateFunc func(ctx context.Context, event datastore.ReplicationEvent, source, target *grpc.ClientConn) error
@@ -711,18 +794,20 @@ func TestProcessBacklog_ReplicatesToReadOnlyPrimary(t *testing.T) {
 		testhelper.DiscardTestEntry(t),
 		conf.VirtualStorageNames(),
 		queue,
+		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
 		&nodes.MockManager{
 			GetShardFunc: func(vs string) (nodes.Shard, error) {
 				require.Equal(t, virtualStorage, vs)
 				return nodes.Shard{
-					IsReadOnly: true,
 					Primary: &nodes.MockNode{
-						StorageName: primaryStorage,
-						Conn:        primaryConn,
+						Healthy:          true,
+						GetStorageMethod: func() string { return primaryStorage },
+						Conn:             primaryConn,
 					},
 					Secondaries: []nodes.Node{&nodes.MockNode{
-						StorageName: secondaryStorage,
-						Conn:        secondaryConn,
+						Healthy:          true,
+						GetStorageMethod: func() string { return secondaryStorage },
+						Conn:             secondaryConn,
 					}},
 				}, nil
 			},
@@ -800,6 +885,7 @@ func newReplicationService(tb testing.TB) (*grpc.Server, string) {
 	gitalypb.RegisterRemoteServiceServer(svr, remote.NewServer(RubyServer))
 	gitalypb.RegisterSSHServiceServer(svr, ssh.NewServer())
 	gitalypb.RegisterRefServiceServer(svr, ref.NewServer())
+	healthpb.RegisterHealthServer(svr, health.NewServer())
 	reflection.Register(svr)
 
 	listener, err := net.Listen("unix", socketName)
@@ -881,7 +967,7 @@ func TestReplMgr_ProcessStale(t *testing.T) {
 	hook := test.NewLocal(logger)
 
 	queue := datastore.NewReplicationEventQueueInterceptor(nil)
-	mgr := NewReplMgr(logger.WithField("test", t.Name()), nil, queue, nil)
+	mgr := NewReplMgr(logger.WithField("test", t.Name()), nil, queue, datastore.NewMemoryRepositoryStore(nil), nil)
 
 	var counter int32
 	queue.OnAcknowledgeStale(func(ctx context.Context, duration time.Duration) error {
