@@ -24,7 +24,6 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/git"
 	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
-	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/datastore"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/grpc-proxy/proxy"
@@ -38,6 +37,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/version"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/health"
 	"google.golang.org/grpc/health/grpc_health_v1"
 	"google.golang.org/grpc/reflection"
 )
@@ -78,7 +78,7 @@ func TestServerRouteServerAccessor(t *testing.T) {
 			"received unexpected request value: %+v instead of %+v", actualReq, expectReq)
 	}()
 
-	ctx, cancel := context.WithTimeout(context.Background(), time.Second)
+	ctx, cancel := testhelper.Context(testhelper.ContextWithTimeout(time.Second))
 	defer cancel()
 
 	actualResp, err := cli.ServerAccessor(ctx, expectReq)
@@ -99,9 +99,8 @@ func TestGitalyServerInfo(t *testing.T) {
 				Name: "virtual-storage-1",
 				Nodes: []*config.Node{
 					&config.Node{
-						Storage:        "praefect-internal-1",
-						DefaultPrimary: true,
-						Token:          "abc",
+						Storage: "praefect-internal-1",
+						Token:   "abc",
 					},
 					&config.Node{
 						Storage: "praefect-internal-2",
@@ -163,10 +162,9 @@ func TestGitalyServerInfoBadNode(t *testing.T) {
 			&config.VirtualStorage{
 				Nodes: []*config.Node{
 					&config.Node{
-						Storage:        "praefect-internal-1",
-						Address:        "unix://" + gitalySocket,
-						DefaultPrimary: true,
-						Token:          "abc",
+						Storage: "praefect-internal-1",
+						Address: "unix://" + gitalySocket,
+						Token:   "abc",
 					},
 				},
 			},
@@ -192,9 +190,8 @@ func TestGitalyDiskStatistics(t *testing.T) {
 			{
 				Nodes: []*config.Node{
 					{
-						Storage:        "praefect-internal-1",
-						DefaultPrimary: true,
-						Token:          "abc",
+						Storage: "praefect-internal-1",
+						Token:   "abc",
 					},
 					{
 						Storage: "praefect-internal-2",
@@ -225,7 +222,7 @@ func TestHealthCheck(t *testing.T) {
 	cc, _, cleanup := runPraefectServerWithGitaly(t, testConfig(1))
 	defer cleanup()
 
-	ctx, cancel := context.WithTimeout(context.Background(), time.Second)
+	ctx, cancel := testhelper.Context(testhelper.ContextWithTimeout(time.Second))
 	defer cancel()
 
 	client := grpc_health_v1.NewHealthClient(cc)
@@ -240,9 +237,8 @@ func TestRejectBadStorage(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp::/this-doesnt-matter",
+						Storage: "praefect-internal-0",
+						Address: "tcp::/this-doesnt-matter",
 					},
 				},
 			},
@@ -273,9 +269,8 @@ func TestWarnDuplicateAddrs(t *testing.T) {
 				Name: "default",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp://abc",
+						Storage: "praefect-internal-0",
+						Address: "tcp://abc",
 					},
 					&config.Node{
 						Storage: "praefect-internal-1",
@@ -287,9 +282,8 @@ func TestWarnDuplicateAddrs(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp://abc",
+						Storage: "praefect-internal-0",
+						Address: "tcp://abc",
 					},
 					&config.Node{
 						Storage: "praefect-internal-1",
@@ -319,9 +313,8 @@ func TestWarnDuplicateAddrs(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp::/samesies",
+						Storage: "praefect-internal-0",
+						Address: "tcp::/samesies",
 					},
 					&config.Node{
 						Storage: "praefect-internal-1",
@@ -356,9 +349,8 @@ func TestWarnDuplicateAddrs(t *testing.T) {
 				Name: "default",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp://abc",
+						Storage: "praefect-internal-0",
+						Address: "tcp://abc",
 					},
 					&config.Node{
 						Storage: "praefect-internal-1",
@@ -370,9 +362,8 @@ func TestWarnDuplicateAddrs(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "tcp://abc",
+						Storage: "praefect-internal-0",
+						Address: "tcp://abc",
 					},
 					&config.Node{
 						Storage: "praefect-internal-2",
@@ -404,9 +395,8 @@ func TestRepoRemoval(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					&config.Node{
-						DefaultPrimary: true,
-						Storage:        gconfig.Config.Storages[0].Name,
-						Address:        "tcp::/samesies",
+						Storage: gconfig.Config.Storages[0].Name,
+						Address: "tcp::/samesies",
 					},
 					&config.Node{
 						Storage: "praefect-internal-1",
@@ -483,11 +473,9 @@ func TestRepoRemoval(t *testing.T) {
 	})
 	require.NoError(t, err)
 
-	resp, err := rClient.RepositoryExists(ctx, &gitalypb.RepositoryExistsRequest{
-		Repository: &virtualRepo,
-	})
-	require.NoError(t, err)
-	require.Equal(t, false, resp.GetExists())
+	storage, ok := gconfig.Config.Storage(conf.VirtualStorages[0].Nodes[0].Storage)
+	require.True(t, ok)
+	testhelper.AssertPathNotExists(t, filepath.Join(storage.Path, tRepo.RelativePath))
 
 	var jobsDone int
 	for {
@@ -528,9 +516,8 @@ func TestRepoRename(t *testing.T) {
 				Name: "praefect",
 				Nodes: []*config.Node{
 					0: {
-						DefaultPrimary: true,
-						Storage:        gconfig.Config.Storages[0].Name,
-						Address:        "tcp::/this-doesnt-matter",
+						Storage: gconfig.Config.Storages[0].Name,
+						Address: "tcp::/this-doesnt-matter",
 					},
 					1: {
 						Storage: "gitaly-1",
@@ -543,6 +530,7 @@ func TestRepoRename(t *testing.T) {
 				},
 			},
 		},
+		Failover: config.Failover{Enabled: true},
 	}
 
 	virtualStorage := conf.VirtualStorages[0]
@@ -743,16 +731,24 @@ func (m *mockSmartHTTP) Called(method string) int {
 
 func newGrpcServer(t *testing.T, srv gitalypb.SmartHTTPServiceServer) (string, *grpc.Server) {
 	socketPath := testhelper.GetTemporaryGitalySocketFileName()
-	grpcSrv, _ := testhelper.NewServerWithHealth(t, socketPath)
+	listener, err := net.Listen("unix", socketPath)
+	require.NoError(t, err)
 
-	gitalypb.RegisterSmartHTTPServiceServer(grpcSrv, srv)
-	reflection.Register(grpcSrv)
+	grpcServer := testhelper.NewTestGrpcServer(t, nil, nil)
 
-	return socketPath, grpcSrv
+	healthSrvr := health.NewServer()
+	grpc_health_v1.RegisterHealthServer(grpcServer, healthSrvr)
+	healthSrvr.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
+	gitalypb.RegisterSmartHTTPServiceServer(grpcServer, srv)
+	reflection.Register(grpcServer)
+
+	go grpcServer.Serve(listener)
+
+	return socketPath, grpcServer
 }
 
 func TestProxyWrites(t *testing.T) {
-	txMgr := transactions.NewManager()
+	txMgr := transactions.NewManager(config.Config{})
 
 	smartHTTP0, smartHTTP1, smartHTTP2 := &mockSmartHTTP{txMgr: txMgr}, &mockSmartHTTP{txMgr: txMgr}, &mockSmartHTTP{txMgr: txMgr}
 
@@ -769,9 +765,8 @@ func TestProxyWrites(t *testing.T) {
 				Name: "default",
 				Nodes: []*config.Node{
 					{
-						DefaultPrimary: true,
-						Storage:        "praefect-internal-0",
-						Address:        "unix://" + socket0,
+						Storage: "praefect-internal-0",
+						Address: "unix://" + socket0,
 					},
 					{
 						Storage: "praefect-internal-1",
@@ -789,10 +784,31 @@ func TestProxyWrites(t *testing.T) {
 	queue := datastore.NewMemoryReplicationEventQueue(conf)
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, queue, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
 	require.NoError(t, err)
+	nodeMgr.Start(0, time.Hour)
 
-	coordinator := NewCoordinator(queue, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	testRepo, _, cleanup := testhelper.NewTestRepo(t)
+	defer cleanup()
+
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+	for _, vs := range conf.VirtualStorages {
+		for _, n := range vs.Nodes {
+			require.NoError(t, rs.SetGeneration(ctx, vs.Name, testRepo.RelativePath, n.Storage, 0))
+		}
+	}
+
+	coordinator := NewCoordinator(
+		queue,
+		rs,
+		nodeMgr,
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	server := grpc.NewServer(
 		grpc.CustomCodec(proxy.NewCodec()),
@@ -808,9 +824,6 @@ func TestProxyWrites(t *testing.T) {
 
 	client, _ := newSmartHTTPClient(t, "unix://"+socket)
 
-	ctx, cancel := testhelper.Context()
-	defer cancel()
-
 	shard, err := nodeMgr.GetShard(conf.VirtualStorages[0].Name)
 	require.NoError(t, err)
 
@@ -820,11 +833,6 @@ func TestProxyWrites(t *testing.T) {
 		waitNodeToChangeHealthStatus(ctx, t, node, true)
 	}
 
-	ctx = featureflag.OutgoingCtxWithFeatureFlags(ctx, featureflag.ReferenceTransactions)
-
-	testRepo, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
 	stream, err := client.PostReceivePack(ctx)
 	require.NoError(t, err)
 
