@@ -62,6 +62,19 @@ func setHeaders(ctx context.Context, repo *gitalypb.Repository, mustExist bool)
 		repoAltDirsHeader, repoAltDirsCombined,
 	)
 
+	// While it looks weird that we're extracting and then re-injecting the
+	// Praefect server info into the context, `PraefectFromContext()` will
+	// also resolve connection information from the context's peer info.
+	// Thus the re-injected connection info will contain resolved addresses.
+	if praefectServer, err := praefect_metadata.PraefectFromContext(ctx); err == nil {
+		ctx, err = praefectServer.Inject(ctx)
+		if err != nil {
+			return nil, err
+		}
+	} else if err != praefect_metadata.ErrPraefectServerNotFound {
+		return nil, err
+	}
+
 	// list of http/2 headers that will be forwarded as-is to gitaly-ruby
 	proxyHeaderWhitelist := []string{
 		"gitaly-servers",
