@@ -1,31 +1,30 @@
 package cleanup
 
 import (
-	"net"
+	"os"
 	"testing"
 
+	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/reflection"
-
-	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 )
 
-func runCleanupServiceServer(t *testing.T) (*grpc.Server, string) {
-	serverSocketPath := testhelper.GetTemporaryGitalySocketFileName()
-	grpcServer := testhelper.NewTestGrpcServer(t, nil, nil)
+func TestMain(m *testing.M) {
+	testhelper.Configure()
+	os.Exit(m.Run())
+}
 
-	listener, err := net.Listen("unix", serverSocketPath)
-	if err != nil {
-		t.Fatal(err)
-	}
+func runCleanupServiceServer(t *testing.T) (string, func()) {
+	srv := testhelper.NewServer(t, nil, nil)
 
-	gitalypb.RegisterCleanupServiceServer(grpcServer, NewServer())
-	reflection.Register(grpcServer)
+	gitalypb.RegisterCleanupServiceServer(srv.GrpcServer(), NewServer())
+	reflection.Register(srv.GrpcServer())
 
-	go grpcServer.Serve(listener)
+	require.NoError(t, srv.Start())
 
-	return grpcServer, "unix://" + serverSocketPath
+	return "unix://" + srv.Socket(), srv.Stop
 }
 
 func newCleanupServiceClient(t *testing.T, serverSocketPath string) (gitalypb.CleanupServiceClient, *grpc.ClientConn) {
