@@ -3,15 +3,15 @@ package commit
 import (
 	"context"
 
-	grpc_logrus "github.com/grpc-ecosystem/go-grpc-middleware/logging/logrus"
+	"github.com/grpc-ecosystem/go-grpc-middleware/logging/logrus/ctxlogrus"
 	"gitlab.com/gitlab-org/gitaly/internal/git"
 	"gitlab.com/gitlab-org/gitaly/internal/git/log"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/chunk"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
 
-func sendCommits(ctx context.Context, sender chunk.Sender, repo *gitalypb.Repository, revisionRange []string, paths []string, extraArgs ...git.Option) error {
-	cmd, err := log.GitLogCommand(ctx, repo, revisionRange, paths, extraArgs...)
+func sendCommits(ctx context.Context, sender chunk.Sender, repo *gitalypb.Repository, revisionRange []string, paths []string, options *gitalypb.GlobalOptions, extraArgs ...git.Option) error {
+	cmd, err := log.GitLogCommand(ctx, repo, revisionRange, paths, options, extraArgs...)
 	if err != nil {
 		return err
 	}
@@ -39,7 +39,7 @@ func sendCommits(ctx context.Context, sender chunk.Sender, repo *gitalypb.Reposi
 	if err := cmd.Wait(); err != nil {
 		// We expect this error to be caused by non-existing references. In that
 		// case, we just log the error and send no commits to the `sender`.
-		grpc_logrus.Extract(ctx).WithError(err).Info("ignoring git-log error")
+		ctxlogrus.Extract(ctx).WithError(err).Info("ignoring git-log error")
 	}
 
 	return nil
