@@ -2,11 +2,13 @@ package commit
 
 import (
 	"gitlab.com/gitlab-org/gitaly/internal/service/ref"
+	"gitlab.com/gitlab-org/gitaly/internal/storage"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
 
 type server struct {
 	gitalypb.UnimplementedCommitServiceServer
+	locator storage.Locator
 }
 
 var (
@@ -14,6 +16,6 @@ var (
 )
 
 // NewServer creates a new instance of a grpc CommitServiceServer
-func NewServer() gitalypb.CommitServiceServer {
-	return &server{}
+func NewServer(locator storage.Locator) gitalypb.CommitServiceServer {
+	return &server{locator: locator}
 }
