@@ -1,7 +1,7 @@
 package remote
 
 import (
-	"gitlab.com/gitlab-org/gitaly/internal/connection"
+	"gitlab.com/gitlab-org/gitaly/client"
 	"gitlab.com/gitlab-org/gitaly/internal/rubyserver"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
@@ -10,13 +10,13 @@ type server struct {
 	ruby *rubyserver.Server
 	gitalypb.UnimplementedRemoteServiceServer
 
-	conns *connection.Pool
+	conns *client.Pool
 }
 
 // NewServer creates a new instance of a grpc RemoteServiceServer
 func NewServer(rs *rubyserver.Server) gitalypb.RemoteServiceServer {
 	return &server{
 		ruby:  rs,
-		conns: connection.NewPool(),
+		conns: client.NewPool(),
 	}
 }
