@@ -3,6 +3,7 @@ package smarthttp
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -10,7 +11,9 @@ import (
 	"path/filepath"
 	"strings"
 	"testing"
+	"time"
 
+	"github.com/golang/protobuf/proto"
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/internal/cache"
 	"gitlab.com/gitlab-org/gitaly/internal/config"
@@ -34,7 +37,10 @@ func TestSuccessfulInfoRefsUploadPack(t *testing.T) {
 
 	rpcRequest := &gitalypb.InfoRefsRequest{Repository: testRepo}
 
-	response, err := makeInfoRefsUploadPackRequest(context.Background(), t, serverSocketPath, rpcRequest)
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	response, err := makeInfoRefsUploadPackRequest(ctx, t, serverSocketPath, rpcRequest)
 	require.NoError(t, err)
 	assertGitRefAdvertisement(t, "InfoRefsUploadPack", string(response), "001e# service=git-upload-pack", "0000", []string{
 		"003ef4e6814c3e4e7a0de82a9e7cd20c626cc963a2f8 refs/tags/v1.0.0",
@@ -80,7 +86,10 @@ func TestSuccessfulInfoRefsUploadPackWithGitConfigOptions(t *testing.T) {
 		GitConfigOptions: []string{"transfer.hideRefs=refs"},
 	}
 
-	response, err := makeInfoRefsUploadPackRequest(context.Background(), t, serverSocketPath, rpcRequest)
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	response, err := makeInfoRefsUploadPackRequest(ctx, t, serverSocketPath, rpcRequest)
 	require.NoError(t, err)
 	assertGitRefAdvertisement(t, "InfoRefsUploadPack", string(response), "001e# service=git-upload-pack", "0000", []string{})
 }
@@ -101,7 +110,7 @@ func TestSuccessfulInfoRefsUploadPackWithGitProtocol(t *testing.T) {
 	}
 
 	client, _ := newSmartHTTPClient(t, serverSocketPath)
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 
 	c, err := client.InfoRefsUploadPack(ctx, rpcRequest)
@@ -151,7 +160,7 @@ func TestSuccessfulInfoRefsReceivePack(t *testing.T) {
 
 	rpcRequest := &gitalypb.InfoRefsRequest{Repository: testRepo}
 
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 	c, err := client.InfoRefsReceivePack(ctx, rpcRequest)
 	if err != nil {
@@ -218,7 +227,7 @@ func TestFailureRepoNotFoundInfoRefsReceivePack(t *testing.T) {
 	repo := &gitalypb.Repository{StorageName: "default", RelativePath: "testdata/scratch/another_repo"}
 	rpcRequest := &gitalypb.InfoRefsRequest{Repository: repo}
 
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 	c, err := client.InfoRefsReceivePack(ctx, rpcRequest)
 	if err != nil {
@@ -239,7 +248,7 @@ func TestFailureRepoNotSetInfoRefsReceivePack(t *testing.T) {
 	defer conn.Close()
 	rpcRequest := &gitalypb.InfoRefsRequest{}
 
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 	c, err := client.InfoRefsReceivePack(ctx, rpcRequest)
 	if err != nil {
@@ -271,6 +280,18 @@ func assertGitRefAdvertisement(t *testing.T, rpc, responseBody string, firstLine
 	}
 }
 
+type mockStreamer struct {
+	streamer
+	putStream func(context.Context, *gitalypb.Repository, proto.Message, io.Reader) error
+}
+
+func (ms mockStreamer) PutStream(ctx context.Context, repo *gitalypb.Repository, req proto.Message, src io.Reader) error {
+	if ms.putStream != nil {
+		return ms.putStream(ctx, repo, req, src)
+	}
+	return ms.streamer.PutStream(ctx, repo, req, src)
+}
+
 func TestCacheInfoRefsUploadPack(t *testing.T) {
 	clearCache(t)
 
@@ -282,9 +303,13 @@ func TestCacheInfoRefsUploadPack(t *testing.T) {
 
 	rpcRequest := &gitalypb.InfoRefsRequest{Repository: testRepo}
 
-	ctx := context.Background()
+	ctx, cancel := testhelper.Context()
+	defer cancel()
 
 	assertNormalResponse := func() {
+		ctx, cancel := context.WithTimeout(ctx, 5*time.Second)
+		defer cancel()
+
 		response, err := makeInfoRefsUploadPackRequest(ctx, t, serverSocketPath, rpcRequest)
 		require.NoError(t, err)
 
@@ -315,10 +340,13 @@ func TestCacheInfoRefsUploadPack(t *testing.T) {
 		replacedContents[0], replacedContents[3], replacedContents[1:3],
 	)
 
-	// invalidate cache for repository
-	ender, err := cache.LeaseKeyer{}.StartLease(rpcRequest.Repository)
-	require.NoError(t, err)
-	require.NoError(t, ender.EndLease(setInfoRefsUploadPackMethod(context.Background())))
+	invalidateCacheForRepo := func() {
+		ender, err := cache.LeaseKeyer{}.StartLease(rpcRequest.Repository)
+		require.NoError(t, err)
+		require.NoError(t, ender.EndLease(setInfoRefsUploadPackMethod(ctx)))
+	}
+
+	invalidateCacheForRepo()
 
 	// replaced cache response is no longer valid
 	assertNormalResponse()
@@ -336,6 +364,21 @@ func TestCacheInfoRefsUploadPack(t *testing.T) {
 	_, err = makeInfoRefsUploadPackRequest(ctx, t, serverSocketPath, invalidReq)
 	testhelper.RequireGrpcError(t, err, codes.Internal)
 	testhelper.AssertPathNotExists(t, pathToCachedResponse(t, ctx, invalidReq))
+
+	// if an error occurs while putting stream, it should not interrupt
+	// request from being served
+	happened := false
+	defer func(old streamer) { infoRefCache = old }(infoRefCache)
+	infoRefCache = mockStreamer{
+		streamer: infoRefCache,
+		putStream: func(context.Context, *gitalypb.Repository, proto.Message, io.Reader) error {
+			happened = true
+			return errors.New("oh nos!")
+		},
+	}
+	invalidateCacheForRepo()
+	assertNormalResponse()
+	require.True(t, happened)
 }
 
 func createInvalidRepo(t testing.TB, repo *gitalypb.Repository) func() {
