@@ -18,8 +18,11 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/config"
 	"gitlab.com/gitlab-org/gitaly/internal/git"
 	"gitlab.com/gitlab-org/gitaly/internal/git/hooks"
+	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
 	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
+	pconfig "gitlab.com/gitlab-org/gitaly/internal/praefect/config"
+	"gitlab.com/gitlab-org/gitaly/internal/praefect/metadata"
 	"gitlab.com/gitlab-org/gitaly/internal/service/hook"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
@@ -121,7 +124,7 @@ func TestFailedReceivePackRequestWithGitOpts(t *testing.T) {
 	client, conn := newSmartHTTPClient(t, serverSocketPath)
 	defer conn.Close()
 
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 
 	stream, err := client.PostReceivePack(ctx)
@@ -140,7 +143,9 @@ func TestFailedReceivePackRequestDueToHooksFailure(t *testing.T) {
 	require.NoError(t, err)
 	defer os.RemoveAll(hookDir)
 
-	defer func() { hooks.Override = "" }()
+	defer func(override string) {
+		hooks.Override = override
+	}(hooks.Override)
 	hooks.Override = hookDir
 
 	require.NoError(t, os.MkdirAll(hooks.Path(), 0755))
@@ -269,7 +274,7 @@ func TestFailedReceivePackRequestDueToValidationError(t *testing.T) {
 
 	for _, rpcRequest := range rpcRequests {
 		t.Run(fmt.Sprintf("%v", rpcRequest), func(t *testing.T) {
-			ctx, cancel := context.WithCancel(context.Background())
+			ctx, cancel := testhelper.Context()
 			defer cancel()
 			stream, err := client.PostReceivePack(ctx)
 			require.NoError(t, err)
@@ -353,12 +358,14 @@ func TestPostReceivePackToHooks(t *testing.T) {
 	features, err := testhelper.NewFeatureSets([]featureflag.FeatureFlag{featureflag.GoPostReceiveHook})
 	require.NoError(t, err)
 
-	ctx, cancel := testhelper.Context()
-	defer cancel()
-
 	for _, feature := range features {
-		t.Run(feature.String(), func(t *testing.T) {
-			testPostReceivePackToHooks(t, feature.WithParent(ctx))
+		t.Run("disabled "+feature.String(), func(t *testing.T) {
+			ctx, cancel := testhelper.Context()
+			defer cancel()
+
+			ctx = feature.Disable(ctx)
+
+			testPostReceivePackToHooks(t, ctx)
 		})
 	}
 }
@@ -414,10 +421,6 @@ func testPostReceivePackToHooks(t *testing.T, ctx context.Context) {
 	config.Config.Gitlab.URL = ts.URL
 	config.Config.Gitlab.SecretFile = filepath.Join(tempGitlabShellDir, ".gitlab_shell_secret")
 
-	defer func(override string) {
-		hooks.Override = override
-	}(hooks.Override)
-
 	cwd, err := os.Getwd()
 	require.NoError(t, err)
 	hookDir := filepath.Join(cwd, "../../../ruby", "git-hooks")
@@ -462,7 +465,7 @@ func runSmartHTTPHookServiceServer(t *testing.T) (*grpc.Server, string) {
 	return server, "unix://" + serverSocketPath
 }
 
-func TestPostReceiveWithTransactions(t *testing.T) {
+func TestPostReceiveWithTransactionsViaPraefect(t *testing.T) {
 	defer func(cfg config.Cfg) {
 		config.Config = cfg
 	}(config.Config)
@@ -482,7 +485,7 @@ func TestPostReceiveWithTransactions(t *testing.T) {
 	require.NoError(t, err)
 
 	for _, features := range featureSets {
-		t.Run(fmt.Sprintf("features:%s", features), func(t *testing.T) {
+		t.Run("disabled "+features.String(), func(t *testing.T) {
 			repo, repoPath, cleanup := testhelper.NewTestRepo(t)
 			defer cleanup()
 
@@ -538,7 +541,7 @@ func TestPostReceiveWithTransactions(t *testing.T) {
 
 			ctx, cancel := testhelper.Context()
 			defer cancel()
-			ctx = features.WithParent(ctx)
+			ctx = features.Disable(ctx)
 
 			stream, err := client.PostReceivePack(ctx)
 			require.NoError(t, err)
@@ -552,3 +555,113 @@ func TestPostReceiveWithTransactions(t *testing.T) {
 		})
 	}
 }
+
+type testTransactionServer struct {
+	called int
+}
+
+func (t *testTransactionServer) VoteTransaction(ctx context.Context, in *gitalypb.VoteTransactionRequest) (*gitalypb.VoteTransactionResponse, error) {
+	t.called++
+	return &gitalypb.VoteTransactionResponse{
+		State: gitalypb.VoteTransactionResponse_COMMIT,
+	}, nil
+}
+
+func TestPostReceiveWithReferenceTransactionHook(t *testing.T) {
+	refTransactionServer := &testTransactionServer{}
+
+	gitalyServer := testhelper.NewTestGrpcServer(t, nil, nil)
+	gitalypb.RegisterSmartHTTPServiceServer(gitalyServer, NewServer())
+	gitalypb.RegisterHookServiceServer(gitalyServer, hook.NewServer(hook.GitlabAPIStub, config.Config.Hooks))
+	gitalypb.RegisterRefTransactionServer(gitalyServer, refTransactionServer)
+	reflection.Register(gitalyServer)
+
+	gitalySocketPath := testhelper.GetTemporaryGitalySocketFileName()
+	listener, err := net.Listen("unix", gitalySocketPath)
+	require.NoError(t, err)
+
+	internalListener, err := net.Listen("unix", config.GitalyInternalSocketPath())
+	require.NoError(t, err)
+
+	go gitalyServer.Serve(listener)
+	go gitalyServer.Serve(internalListener)
+	defer gitalyServer.Stop()
+
+	gitlabShellDir, cleanup := testhelper.CreateTemporaryGitlabShellDir(t)
+	defer cleanup()
+
+	defer func(oldValue string) {
+		config.Config.Gitlab.SecretFile = oldValue
+	}(config.Config.Gitlab.SecretFile)
+	config.Config.Gitlab.SecretFile = filepath.Join(gitlabShellDir, ".gitlab_shell_secret")
+	testhelper.WriteShellSecretFile(t, gitlabShellDir, "secret123")
+
+	defer func(override string) {
+		hooks.Override = override
+	}(hooks.Override)
+	hooks.Override = ""
+
+	featureSets, err := testhelper.NewFeatureSets([]featureflag.FeatureFlag{
+		featureflag.ReferenceTransactionHook,
+	})
+	require.NoError(t, err)
+
+	for _, features := range featureSets {
+		t.Run("disabled "+features.String(), func(t *testing.T) {
+			if !features.IsDisabled(featureflag.ReferenceTransactionHook) {
+				t.Skip("reference-transaction hook is broken in git-core")
+			}
+
+			refTransactionServer.called = 0
+
+			client, conn := newSmartHTTPClient(t, "unix://"+gitalySocketPath)
+			defer conn.Close()
+
+			ctx, cancel := testhelper.Context()
+			defer cancel()
+
+			ctx, err = metadata.InjectTransaction(ctx, 1234, "primary", true)
+			require.NoError(t, err)
+
+			// As we ain't got a Praefect server setup, we instead hooked up the
+			// RefTransaction server for Gitaly itself. As this is the only Praefect
+			// service required in this context, we can just pretend that
+			// Gitaly is the Praefect server and inject it.
+			praefectServer, err := metadata.PraefectFromConfig(pconfig.Config{
+				SocketPath: "unix://" + gitalySocketPath,
+			})
+			require.NoError(t, err)
+
+			ctx, err = praefectServer.Inject(ctx)
+			require.NoError(t, err)
+
+			ctx = helper.IncomingToOutgoing(ctx)
+			ctx = features.Disable(ctx)
+
+			stream, err := client.PostReceivePack(ctx)
+			require.NoError(t, err)
+
+			repo, _, cleanup := testhelper.NewTestRepo(t)
+			defer cleanup()
+
+			request := &gitalypb.PostReceivePackRequest{Repository: repo, GlId: "key-1234", GlRepository: "some_repo"}
+			response := doPush(t, stream, request, newTestPush(t, nil).body)
+
+			expectedResponse := "0030\x01000eunpack ok\n0019ok refs/heads/master\n00000000"
+			require.Equal(t, expectedResponse, string(response), "Expected response to be %q, got %q", expectedResponse, response)
+
+			version, err := git.Version()
+			require.NoError(t, err)
+			supported, err := git.SupportsReferenceTransactionHook(version)
+			require.NoError(t, err)
+
+			// If the reference-transaction hook is not supported or the feature flag is
+			// not enabled, voting only happens via the pre-receive hook.
+			if features.IsDisabled(featureflag.ReferenceTransactionHook) || !supported {
+				require.Equal(t, 1, refTransactionServer.called)
+			} else {
+				require.Equal(t, 3, refTransactionServer.called)
+			}
+		})
+	}
+}
