@@ -135,24 +135,6 @@ describe Gitlab::Git::Repository do # rubocop:disable Metrics/BlockLength
     end
   end
 
-  describe '#fetch_repository_as_mirror' do
-    let(:new_repository) { gitlab_git_from_gitaly(new_empty_test_repo) }
-    let(:repository) { mutable_repository }
-    let(:remote_repository) { Gitlab::Git::RemoteRepository.new(repository) }
-    let(:fake_env) { {} }
-
-    subject { new_repository.fetch_repository_as_mirror(remote_repository) }
-
-    it 'fetches a repository as a mirror remote' do
-      expect(new_repository).to receive(:add_remote).with(anything, Gitlab::Git::Repository::GITALY_INTERNAL_URL, mirror_refmap: :all_refs)
-      expect(remote_repository).to receive(:fetch_env).and_return(fake_env)
-      expect(new_repository).to receive(:fetch_remote).with(anything, env: fake_env)
-      expect(new_repository).to receive(:remove_remote).with(anything)
-
-      subject
-    end
-  end
-
   describe '#merge_base' do
     where(:from, :to, :result) do
       '570e7b2abdd848b95f2f578043fc23bd6f6fd24d' | '40f4a7a617393735a95a0bb67b08385bc1e7c66d' | '570e7b2abdd848b95f2f578043fc23bd6f6fd24d'
@@ -608,16 +590,51 @@ describe Gitlab::Git::Repository do # rubocop:disable Metrics/BlockLength
     end
   end
 
+  describe '#rebase' do
+    let(:repository) { mutable_repository }
+    let(:rebase_id) { '2' }
+    let(:branch_name) { 'rd-add-file-larger-than-1-mb' }
+    let(:branch_sha) { 'c54ad072fabee9f7bf9b2c6c67089db97ebfbecd' }
+    let(:remote_branch) { 'master' }
+
+    subject do
+      opts = {
+        branch: branch_name,
+        branch_sha: branch_sha,
+        remote_repository: repository,
+        remote_branch: remote_branch
+      }
+
+      repository.rebase(user, rebase_id, opts)
+    end
+
+    describe 'sparse checkout' do
+      let(:expected_files) { %w[files/images/emoji.png] }
+
+      it 'lists files modified in source branch in sparse-checkout' do
+        allow(repository).to receive(:with_worktree).and_wrap_original do |m, *args|
+          m.call(*args) do
+            worktree = args[0]
+            sparse = repository.path + "/worktrees/#{worktree.name}/info/sparse-checkout"
+            diff_files = IO.readlines(sparse, chomp: true)
+
+            expect(diff_files).to eq(expected_files)
+          end
+        end
+
+        subject
+      end
+    end
+  end
+
   describe '#squash' do
     let(:repository) { mutable_repository }
     let(:squash_id) { '1' }
-    let(:branch_name) { 'fix' }
     let(:start_sha) { '4b4918a572fa86f9771e5ba40fbd48e1eb03e2c6' }
     let(:end_sha) { '12d65c8dd2b2676fa3ac47d955accc085a37a9c1' }
 
     subject do
       opts = {
-        branch: branch_name,
         start_sha: start_sha,
         end_sha: end_sha,
         author: user,
@@ -668,6 +685,21 @@ describe Gitlab::Git::Repository do # rubocop:disable Metrics/BlockLength
           subject
         end
       end
+
+      describe 'when worktree throws an error' do
+        before do
+          allow(repository).to receive(:run_git).and_return(['ok', 0])
+          expect(repository).to receive(:run_git)
+            .with(array_including('worktree'), chdir: anything, env: anything, nice: false, include_stderr: true, lazy_block: anything)
+            .and_return(['error', 1])
+        end
+
+        it 'includes the stderr output' do
+          expect do
+            subject
+          end.to raise_error(described_class::GitError, 'error')
+        end
+      end
     end
 
     describe 'with an ASCII-8BIT diff' do
